from marshmallow import Schema, fields, validate, ValidationError

from mobio.libs.profiling_mf import ProfileByIdentifyStructure
from mobio.libs.profiling_mf.merge_fields.base_merge import BaseMerge, MergeListGroup
from mobio.libs.profiling_mf.profiling_common import DisplayType, ProfileHistoryChangeType
from mobio.libs.profiling_mf.profiling_schema import ProfileByIdentifySchema


class MergeProfileIdentify(BaseMerge):
    def serialize_data(
        self,
        suggest_data,
        profile_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
        predict=None
    ):
        lst_profile_identify = []
        profile_data = profile_data if profile_data is not None else []
        for profile_identify in profile_data:
            lst_profile_identify.append(
                self.__build_value__(
                    value=profile_identify, suggest=True, changealbe=False, predict=predict
                )
            )
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=True,
            editable=False,
            mergeable=True,
            order=1,
            group=MergeListGroup.INFORMATION,
            value=lst_profile_identify,
        )

    def set_filter_value(self, suggest_filter_data, profile_data):
        pass

    def serialize_origin_data(
        self,
        suggest_data,
        origin_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
    ):
        lst_profile_identify = []
        origin_data = origin_data if origin_data is not None else []
        for profile_identify in origin_data:
            # profile_identify_unique_value = "{}:{}".format(
            #     profile_identify.get(ProfileByIdentifyStructure.IDENTIFY_TYPE),
            #     profile_identify.get(ProfileByIdentifyStructure.IDENTIFY_VALUE),
            # )
            # suggest_profile_identify = (
            #     True
            #     if profile_identify_unique_value not in set_unique_suggest_values
            #     else False
            # )
            # set_unique_suggest_values.add(profile_identify_unique_value)
            lst_profile_identify.append(
                self.__build_value__(
                    value=profile_identify, suggest=True, changealbe=False
                )
            )
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=True,
            editable=False,
            mergeable=True,
            order=1,
            group=MergeListGroup.INFORMATION,
            value=lst_profile_identify,
        )

    def merge_data(self, target_data, source_data, field_key, is_master_data=False):
        if source_data:
            # normalize data
            target_data[field_key] = (
                target_data.get(field_key) if target_data.get(field_key) else []
            )
            # set data is empty if is_master_data is True
            target_data[field_key] = (
                target_data.get(field_key) if not is_master_data else []
            )
            for source_identify in source_data.get("field_value"):
                suggest = source_identify.get("suggest")
                if suggest:
                    profile_identify = ProfileByIdentifySchema().load(
                        source_identify.get("value")
                    )

                    exists_identify = next(
                        (
                            x
                            for x in target_data.get(field_key)
                            if x.get(ProfileByIdentifyStructure.IDENTIFY_TYPE)
                            == profile_identify.get(
                                ProfileByIdentifyStructure.IDENTIFY_TYPE
                            )
                            and x.get(ProfileByIdentifyStructure.IDENTIFY_VALUE)
                            == profile_identify.get(
                                ProfileByIdentifyStructure.IDENTIFY_VALUE
                            )
                        ),
                        None,
                    )
                    if not exists_identify:
                        target_data[field_key].append(profile_identify)

    def validate_merge(self, data, schema_validate_value=None):
        schema_validate_value = {
            "changeable": fields.Boolean(default=True, missing=True, allow_none=False),
            "suggest": fields.Boolean(default=True, missing=True, allow_none=False),
            "value": fields.Nested(ProfileByIdentifySchema),
            "status": fields.Int(default=None, allow_none=True),
            "predict": fields.Dict(allow_none=True)
        }
        field_value = data.get("field_value", [])
        tmp_field_value = []
        for value in field_value:
            generated_schema_value = Schema.from_dict(schema_validate_value)
            try:
                valid_value = generated_schema_value().load(value)
                tmp_field_value.append(valid_value)
            except ValidationError as ex:
                print(
                    "MergeProfileIdentify:: validation value error: {}".format(ex)
                )
        data["field_value"] = tmp_field_value
        rules = {
            "display_type": fields.Str(
                validate=validate.OneOf([x.value for x in DisplayType])
            ),
            "displayable": fields.Boolean(),
            "editable": fields.Boolean(),
            "field_property": fields.Int(),
            "group": fields.Str(),
            "mergeable": fields.Boolean(),
            "order": fields.Int(),
            "tooltip_i18n": fields.Str(allow_none=True, missing=None, default=None),
            "translate_key": fields.Str(allow_none=True, missing=None, default=None),
            "field_value": fields.List(
                fields.Nested(Schema.from_dict(schema_validate_value))
            ),
        }

        generated_schema = Schema.from_dict(rules)
        return generated_schema().load(data)

    def get_add_data(self, data):
        if not data:
            data = dict()
        if ProfileHistoryChangeType.ADD not in data:
            data[ProfileHistoryChangeType.ADD] = []
        return [x.get(ProfileByIdentifyStructure.IDENTIFY_VALUE) for x in data.get(ProfileHistoryChangeType.ADD)]

    def get_remove_data(self, data):
        if not data:
            data = dict()
        if ProfileHistoryChangeType.REMOVE not in data:
            data[ProfileHistoryChangeType.REMOVE] = []
        return [x.get(ProfileByIdentifyStructure.IDENTIFY_VALUE) for x in data.get(ProfileHistoryChangeType.REMOVE)]



def df_get_operation_data():
    lst = []
    operation_data = [
        {
            "id": 1,
            "name": "An ninh - Bảo vệ"
        },
        {
            "id": 2,
            "name": "Báo chí - Truyền hình"
        },
        {
            "id": 3,
            "name": "Bảo hiểm"
        },
        {
            "id": 4,
            "name": "Phiên dịch"
        },
        {
            "id": 5,
            "name": "Phim"
        },
        {
            "id": 6,
            "name": "Cơ khí chế tạo máy"
        },
        {
            "id": 7,
            "name": "Thế thao"
        },
        {
            "id": 8,
            "name": "Y tế"
        },
        {
            "id": 9,
            "name": "Giáo dục"
        },
        {
            "id": 10,
            "name": "Ẩm thực"
        },
        {
            "id": 11,
            "name": "Khoa học - Kỹ thuật"
        },
        {
            "id": 12,
            "name": "Địa chất"
        },
        {
            "id": 13,
            "name": "Môi trường"
        },
        {
            "id": 14,
            "name": "Nông nghiệp"
        }
    ]
    for operation in operation_data:
        operation["display_in_form"] = True
        lst.append(operation)
    return lst



from mobio.libs.profiling_mf.merge_fields.base_merge import BaseMerge, MergeListGroup
from mobio.libs.profiling_mf.profiling_common import ProfileHistoryChangeType


class MergeDistrictCode(BaseMerge):
    def serialize_data(
        self,
        suggest_data,
        profile_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
        predict=None
    ):
        suggest_province = False
        if profile_data is not None and field_key not in set_suggest_fields:
            suggest_province = True
            set_suggest_fields.add(field_key)

        field_value = self.__build_value__(
            value=profile_data.get("id") if profile_data is not None else None,
            suggest=suggest_province,
            predict=predict
        )
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=True,
            editable=True,
            mergeable=True,
            order=1,
            group=MergeListGroup.DEMOGRAPHIC,
            value=field_value,
        )

    def set_filter_value(self, suggest_filter_data, profile_data):
        pass

    def serialize_origin_data(
        self,
        suggest_data,
        origin_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
    ):
        suggest = True if field_key not in set_suggest_fields and origin_data else False
        field_value = self.__build_value__(
            value=origin_data,
            suggest=suggest,
        )
        if suggest:
            set_suggest_fields.add(field_key)
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=True,
            editable=True,
            mergeable=True,
            order=1,
            group=MergeListGroup.DEMOGRAPHIC,
            value=field_value,
        )

    def merge_data(self, target_data, source_data, field_key, is_master_data=False):
        if source_data:
            suggest = source_data.get("field_value").get("suggest")
            value = source_data.get("field_value").get("value")
            # if suggest and value is not None:
            #     district_code = next(
            #         (x.value for x in [x for x in DistrictEnum] if x.value.code == value), None
            #     )
            #     if district_code:
            #         target_data[field_key] = CommonHelper.create_simple_data_type(
            #             _id=district_code.code, _name=district_code.name
            #         )

    def get_updated_data(self, data):
        if not data:
            data = dict()
        if data.get(ProfileHistoryChangeType.REMOVE):
            old_value = data.get(ProfileHistoryChangeType.REMOVE)[0]
        else:
            old_value = (
                data.get(ProfileHistoryChangeType.CHANGE)[0].get("from")
                if data.get(ProfileHistoryChangeType.CHANGE)
                else None
            )
        if data.get(ProfileHistoryChangeType.ADD):
            new_value = data.get(ProfileHistoryChangeType.ADD)[0]
        else:
            new_value = (
                data.get(ProfileHistoryChangeType.CHANGE)[0].get("to")
                if data.get(ProfileHistoryChangeType.CHANGE)
                else None
            )
        return old_value if old_value else None, new_value if new_value else None



from mobio.libs.profiling_mf import ProfileStructure
from mobio.libs.profiling_mf.merge_fields.base_merge import BaseMerge, MergeListGroup


class MergeAddress(BaseMerge):
    def serialize_data(
        self,
        suggest_data,
        profile_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
        predict=None
    ):
        set_address = set()
        lst_suggest_address = []
        profile_data = profile_data if profile_data is not None else []
        for a in profile_data:
            if a not in set_address:
                lst_suggest_address.append(
                    self.__build_value__(value=a, suggest=True, changealbe=True, predict=predict)
                )
                set_address.add(a)
        suggest_data[ProfileStructure.PROFILE_ADDRESS] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=True,
            editable=True,
            mergeable=True,
            order=1,
            group=MergeListGroup.DEMOGRAPHIC,
            value=lst_suggest_address,
        )

    def set_filter_value(self, suggest_filter_data, profile_data):
        pass

    def serialize_origin_data(
        self,
        suggest_data,
        origin_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
    ):
        set_address = set()
        lst_suggest_address = []
        origin_data = origin_data if origin_data else []
        for a in origin_data:
            if a not in set_address:
                lst_suggest_address.append(
                    self.__build_value__(value=a, suggest=True, changealbe=True)
                )
                set_address.add(a)
        suggest_data[ProfileStructure.PROFILE_ADDRESS] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=True,
            editable=True,
            mergeable=True,
            order=1,
            group=MergeListGroup.DEMOGRAPHIC,
            value=lst_suggest_address,
        )

    def merge_data(self, target_data, source_data, field_key, is_master_data=False):
        target_data[ProfileStructure.ADDRESS] = (
            target_data.get(ProfileStructure.ADDRESS)
            if target_data.get(ProfileStructure.ADDRESS)
            else []
        )
        set_address = (
            set(target_data.get(ProfileStructure.ADDRESS))
            if not is_master_data
            else set()
        )
        if source_data:
            for address_suggest in source_data.get("field_value"):
                suggest = address_suggest.get("suggest")
                value = (
                    str(address_suggest.get("value")).strip()
                    if address_suggest.get("value") is not None
                    else ""
                )
                if suggest and value:
                    set_address.add(value)
        target_data[ProfileStructure.ADDRESS] = list(set_address)



from marshmallow import Schema, fields, validate

from mobio.libs.profiling_mf.profiling_common import SocialType, SocialIdType


class SocialUserSchema(Schema):
    social_id = fields.Str(required=True)
    social_type = fields.Int(
        required=True,
        validate=validate.OneOf(
            [
                SocialType.FACEBOOK,
                SocialType.ZALO,
                SocialType.INSTAGRAM,
                SocialType.YOUTUBE,
                SocialType.APP,
                SocialType.LINE,
                SocialType.MOBIO_CHAT_TOOL,
            ]
        ),
    )
    access_token = fields.Str(allow_none=True, default=None)
    social_id_type = fields.Int(
        allow_none=True, default=SocialIdType.PAGE, missing=SocialIdType.PAGE
    )
    social_name = fields.Str(allow_none=True, default=None)
    page_id = fields.Str(allow_none=True, default=None, missing=None)
    app_id = fields.Str(allow_none=True, default=None, missing=None)


class VibCardSchema(Schema):
    account_serno = fields.Str(required=True)
    annual_fees_next_action_date = fields.DateTime(allow_none=True, missing=None)
    card_id = fields.Str(required=True)
    card_limit = fields.Float(required=True)
    card_name = fields.Str(required=True)
    card_no = fields.Str(required=True)
    card_open_date = fields.DateTime(required=True)
    card_type = fields.Str(required=True)
    card_type_id = fields.Str(required=True)
    is_primary_card = fields.Boolean(required=True)
    status = fields.Str(required=True)


class UdtSchema(Schema):
    id = fields.Int(required=True, allow_none=False)
    value = fields.Str(required=True)


class HobbySchema(Schema):
    id = fields.Int(required=True, allow_none=False)
    name = fields.Str(required=True)


class PushIdSchema(Schema):
    push_id = fields.Str(required=True)
    os_type = fields.Int(required=True)
    app_id = fields.Str(required=True)
    lang = fields.Str(required=False, allow_none=True, missing="vi", default="vi")
    is_logged = fields.Boolean(required=False, allow_none=True, missing=False, default=False)
    last_access = fields.DateTime(required=False, allow_none=True, missing=None, default=None)
    count_fail = fields.Int(required=False, allow_none=True, missing=0, default=0)
    device_id = fields.Str(required=False, allow_none=True, missing=None, default=None)
    accept_push = fields.Boolean(required=False, allow_none=True, missing=True, default=True)


class CardSchema(Schema):
    id = fields.Str(required=True, allow_none=False)
    card_id = fields.Str(required=True, allow_none=False)
    card_name = fields.Str(required=True, allow_none=False)
    card_code = fields.Str(required=True, allow_none=False)
    is_primary = fields.Boolean(required=False, default=False, missing=False, allow_none=True)
    card_status = fields.Int(required=True, allow_none=False)
    expiry_time = fields.DateTime(required=False, allow_none=True)


class ProfileTagSchema(Schema):
    id = fields.UUID(required=True)
    merchant_id = fields.UUID(required=True)
    tag = fields.Str(required=True)
    tag_type = fields.Int(required=True)


class DeviceTypeSchema(Schema):
    device_type = fields.Str(required=True, allow_none=False)
    device_name = fields.Str(required=True, allow_none=False)


class DeviceSchema(Schema):
    device_id = fields.Str(required=True, validate=validate.Length(min=3))
    device_name = fields.Str(allow_none=True)
    source = fields.Str(required=True, validate=validate.Length(min=3))


class ProfileByIdentifySchema(Schema):
    merchant_id = fields.List(fields.UUID, allow_none=True)
    profile_id = fields.UUID(allow_none=True)
    identify_value = fields.Str(required=True, validate=validate.Length(min=3))
    identify_type = fields.Str(required=True, validate=validate.Length(min=3))
    is_verify = fields.Boolean(allow_none=True, missing=False)
    date_verify = fields.DateTime(allow_none=True, missing=None)
    verify_by = fields.Str(allow_none=True, missing=None)



from mobio.libs.profiling_mf import ProfileStructure
from mobio.libs.profiling_mf.common_helper import CommonHelper
from mobio.libs.profiling_mf.merge_fields.base_merge import BaseMerge, MergeListGroup
from mobio.libs.profiling_mf.profiling_common import ProfileHistoryChangeType


class MergeSecondaryPhones(BaseMerge):
    def serialize_data(
        self,
        suggest_data,
        profile_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
        predict=None
    ):
        lst_phone_2 = []
        if (
            type(profile_data) == dict
            and field_key == ProfileStructure.SECONDARY_PHONES
        ):
            for phone in profile_data.get("secondary"):
                # suggest = (
                #     True
                #     if phone.get(ProfileStructure.PHONE_NUMBER)
                #     not in set_unique_suggest_values
                #     else False
                # )
                status = phone.get("status")
                value = phone.get(ProfileStructure.PHONE_NUMBER)
                field_value = self.__build_value__(
                    value=value, suggest=True, changealbe=False
                )
                field_value["status"] = status
                lst_phone_2.append(field_value)
                # set_unique_suggest_values.add(phone.get(ProfileStructure.PHONE_NUMBER))

            suggest_data[field_key] = self.build_merge_data(
                translate_key=translate_key,
                field_property=field_property,
                display_type=display_type,
                displayable=True,
                editable=False,
                mergeable=True,
                order=1,
                group=MergeListGroup.INFORMATION,
                value=lst_phone_2,
            )

    def set_filter_value(self, suggest_filter_data: set, profile_data):
        if type(profile_data) == dict:
            for phone in profile_data.get("secondary"):
                suggest_filter_data.add(phone.get(ProfileStructure.PHONE_NUMBER))

    def serialize_origin_data(
        self,
        suggest_data,
        origin_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
    ):
        if origin_data:
            lst_phone_2 = []
            if type(origin_data) == list:
                for phone in origin_data:
                    phone_valid = CommonHelper().chuan_hoa_so_dien_thoai_v2(phone)
                    if phone_valid:
                        field_value = self.__build_value__(
                            value=phone_valid, suggest=True, changealbe=False
                        )
                        lst_phone_2.append(field_value)
                        # set_unique_suggest_values.add(phone_valid)
            elif type(origin_data) == str:
                phone_valid = CommonHelper().chuan_hoa_so_dien_thoai_v2(origin_data)
                if phone_valid:
                    field_value = self.__build_value__(
                        value=phone_valid, suggest=True, changealbe=False
                    )
                    lst_phone_2.append(field_value)
                    # set_unique_suggest_values.add(phone_valid)
            else:
                print('key: {}, origin_data: {} is not valid'.format(field_key, origin_data))
            suggest_data[ProfileStructure.SECONDARY_PHONES] = self.build_merge_data(
                translate_key=translate_key,
                field_property=field_property,
                display_type=display_type,
                displayable=True,
                editable=False,
                mergeable=True,
                order=1,
                group=MergeListGroup.INFORMATION,
                value=lst_phone_2,
            )

    def merge_data(self, target_data, source_data, field_key, is_master_data=False):
        if source_data:
            primary_phone = (
                target_data.get(ProfileStructure.PRIMARY_PHONE).get(
                    ProfileStructure.PHONE_NUMBER
                )
                if target_data.get(ProfileStructure.PRIMARY_PHONE)
                else None
            )
            # normalize lại data
            target_data[ProfileStructure.PHONE_NUMBER] = (
                target_data.get(ProfileStructure.PHONE_NUMBER)
                if target_data.get(ProfileStructure.PHONE_NUMBER)
                else []
            )
            # set_phone = [] khi đây là master_data, nếu không thì set_data sẽ là tất cả phone
            set_phone = set(
                target_data.get(ProfileStructure.PHONE_NUMBER) if not is_master_data else []
            )
            # add primary_phone vào set_phone để tránh bị mất primary_phone
            if primary_phone:
                set_phone.add(primary_phone)
            for source_phone in source_data.get("field_value"):
                phone = source_phone.get("value")
                suggest = source_phone.get("suggest")
                phone_valid = CommonHelper().chuan_hoa_so_dien_thoai_v2(phone)
                if phone_valid and suggest:
                    set_phone.add(phone_valid)
            target_data[ProfileStructure.PHONE_NUMBER] = list(set_phone)
            CommonHelper.set_phone_profile(profile=target_data, primary_phone=primary_phone)

    def get_add_data(self, data):
        if not data:
            data = dict()
        if ProfileHistoryChangeType.ADD not in data:
            data[ProfileHistoryChangeType.ADD] = []
        return [x.get(ProfileStructure.PHONE_NUMBER) for x in data.get(ProfileHistoryChangeType.ADD)]

    def get_remove_data(self, data):
        if not data:
            data = dict()
        if ProfileHistoryChangeType.REMOVE not in data:
            data[ProfileHistoryChangeType.REMOVE] = []
        return [x.get(ProfileStructure.PHONE_NUMBER) for x in data.get(ProfileHistoryChangeType.REMOVE)]


from mobio.libs.profiling_mf.merge_fields.base_merge import BaseMerge, MergeListGroup


class MergeIncomeType(BaseMerge):
    def serialize_data(
        self,
        suggest_data,
        profile_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
        predict=None
    ):
        suggest = (
            True
            if profile_data is not None and field_key not in set_suggest_fields
            else False
        )
        if suggest:
            set_suggest_fields.add(field_key)
        field_value = self.__build_value__(value=profile_data, suggest=suggest, predict=predict)
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=False,
            editable=False,
            mergeable=True,
            order=1,
            group=MergeListGroup.DYNAMIC,
            value=field_value,
        )

    def set_filter_value(self, suggest_filter_data, profile_data):
        pass

    def serialize_origin_data(
        self,
        suggest_data,
        origin_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
    ):
        suggest = True if field_key not in set_suggest_fields and origin_data else False
        field_value = self.__build_value__(value=origin_data, suggest=suggest)
        if suggest:
            set_suggest_fields.add(field_key)
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=False,
            editable=False,
            mergeable=True,
            order=1,
            group=MergeListGroup.DYNAMIC,
            value=field_value,
        )



import inspect
from enum import Enum


class SocialIdType:
    APP = 1
    GLOBAL = 2
    PAGE = 3


class SocialType:
    FACEBOOK = 1
    ZALO = 2
    INSTAGRAM = 3
    YOUTUBE = 4
    APP = 5
    LINE = 6
    MOBIO_CHAT_TOOL = 7


class DynamicFieldProperty:
    INTEGER = 1
    FLOAT = 4
    STRING = 2
    DATETIME = 3
    DICT = 5
    EMAIL = 6
    PHONE_NUMBER = 7
    GENDER = 8
    RELATIONSHIP_DATA = 9
    RELATION_WITH_CHILDS = 10
    CHILDS = 11
    SOCIAL_TAGS = 12
    SOCIAL_USER = 13
    UDT = 14
    CARDS = 15
    PUSH_ID = 16
    PS = 17

    @classmethod
    def get_all_property(cls):
        attributes = inspect.getmembers(cls, lambda a: not (inspect.isroutine(a)))
        values = [
            a[1]
            for a in attributes
            if not (a[0].startswith("__") and a[0].endswith("__"))
        ]

        print(
            "DynamicFieldProperty::get_all_property: values = %s" % values
        )
        return values


class DisplayType(Enum):
    SINGLE_LINE = "single_line"
    MULTI_LINE = "multi_line"
    DROPDOWN_SINGLE_LINE = "dropdown_single_line"
    DROPDOWN_MULTI_LINE = "dropdown_multi_line"
    RADIO_SELECT = "radio"
    CHECKBOX = "checkbox"
    DATE_PICKER = "date_picker"
    TAGS = "tags"


class Group:
    INFORMATION = "information"
    DEMOGRAPHIC = "demographic"
    ACTIVITY = "activity"
    LOYALTY = "loyalty"
    OTHER = "other"
    DYNAMIC = "dynamic"


DISPLAY_TYPE_IS_LIST_TYPE = [
    DisplayType.MULTI_LINE.value,
    DisplayType.DROPDOWN_MULTI_LINE.value,
    DisplayType.CHECKBOX.value,
    DisplayType.TAGS.value,
]


class EmailValidateStatus:
    Invalid = -1
    Uncheck = 0
    Valid = 1


class PhoneVerifyStatus:
    VerifyFailed = -1
    UnVerify = 0
    Verified = 1


class ProfileHistoryChangeType:
    ADD = "add"
    REMOVE = "remove"
    CHANGE = "change"



def df_get_account_type_data():
    data = [
        {"id": -1, "name": "Khác"},
        {"id": 0, "name": "Nhập Thủ Công"},
        {"id": 1, "name": "Số Điện Thoại"},
        {"id": 2, "name": "Facebook"},
        {"id": 3, "name": "G+"},
        {"id": 4, "name": "Landing Page"},
        {"id": 5, "name": "Nhập Từ File"},
        {"id": 6, "name": "Zalo"},
        {"id": 7, "name": "Instagram"},
        {"id": 8, "name": "Mobile App"},
        {"id": 9, "name": "Youtube"},
        {"id": 10, "name": "Zalo"},
        {"id": 11, "name": "Smart Wifi"},
        {"id": 12, "name": "Call Center"},
        {"id": 13, "name": "Social"},
        {"id": 15, "name": "Phong Vũ Odoo"},
        {"id": 16, "name": "Phong Vũ Magento"},
        {"id": 17, "name": "Phong Vũ Inhouse"},
        {"id": 18, "name": "Phong Vũ Asia"},
        {"id": 19, "name": "Face ID"},
        {"id": 20, "name": "Line"},
        {"id": 21, "name": "Website"},
        {"id": 22, "name": "POS"},
        {"id": 23, "name": "Core"},
    ]
    return data


def get_account_type_by_id(account_type_id):
    data_account_type = df_get_account_type_data()
    obj_data = next((x for x in data_account_type if x.get("id") == account_type_id), None)
    return obj_data.get("name") if obj_data else None



from abc import ABCMeta, abstractmethod
from marshmallow import Schema, fields, validate
from mobio.libs.profiling_mf.profiling_common import DynamicFieldProperty, DisplayType, DISPLAY_TYPE_IS_LIST_TYPE, \
    ProfileHistoryChangeType
from mobio.libs.profiling_mf.profiling_schema import SocialUserSchema, PushIdSchema, CardSchema


class MergeListGroup:
    INFORMATION = "information"
    DEMOGRAPHIC = "demographic"
    ACTIVITY = "activity"
    LOYALTY = "loyalty"
    OTHER = "other"
    DYNAMIC = "dynamic"


class BaseMerge(metaclass=ABCMeta):
    @staticmethod
    def build_merge_data(
        translate_key,
        field_property,
        display_type,
        displayable,
        editable,
        mergeable,
        order,
        group,
        value,
        tooltip_i18n=None
    ):
        return {
            "translate_key": translate_key,
            "tooltip_i18n": tooltip_i18n,
            "field_property": field_property,
            "display_type": display_type,
            "displayable": displayable,
            "editable": editable,
            "mergeable": mergeable,
            "order": order,
            "group": group,
            "field_value": value,
        }

    @abstractmethod
    def serialize_data(
        self,
        suggest_data,
        profile_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
        predict=None
    ):
        pass

    @abstractmethod
    def set_filter_value(self, suggest_filter_data, profile_data):
        pass

    @abstractmethod
    def serialize_origin_data(
        self,
        suggest_data,
        origin_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
    ):
        pass

    def merge_data(self, target_data, source_data, field_key, is_master_data=False):
        if source_data:
            suggest = source_data.get("field_value").get("suggest")
            value = source_data.get("field_value").get("value")
            if suggest and value is not None:
                target_data[field_key] = value

    def increase_trust_point(self, **kwargs):
        return 0

    def __build_value__(self, value, suggest=True, changealbe=True, predict=None):
        return {"value": value, "suggest": suggest, "changeable": changealbe, "predict": predict}

    def __validate_field_value__(self, field_property, validate_value=None):
        if not validate_value:
            if field_property in [
                DynamicFieldProperty.INTEGER,
                DynamicFieldProperty.GENDER,
                DynamicFieldProperty.UDT
            ]:
                validate_value = fields.Int(allow_none=True)
            elif field_property == DynamicFieldProperty.FLOAT:
                validate_value = fields.Float(allow_none=True)
            elif field_property == DynamicFieldProperty.DATETIME:
                validate_value = fields.DateTime(allow_none=True)
            elif field_property in [
                DynamicFieldProperty.STRING,
                DynamicFieldProperty.PHONE_NUMBER,
            ]:
                validate_value = fields.Str(allow_none=True)
            elif field_property == DynamicFieldProperty.DICT:
                validate_value = fields.Dict(allow_none=True)
            elif field_property == DynamicFieldProperty.EMAIL:
                validate_value = fields.Email(allow_none=True)
            elif field_property == DynamicFieldProperty.SOCIAL_USER:
                validate_value = fields.Nested(
                    SocialUserSchema, allow_none=True, missing=None, default=None
                )
            # elif field_property == DynamicFieldProperty.UDT:
            #     validate_value = fields.Nested(
            #         UdtSchema, allow_none=True, missing=None, default=None
            #     )
            elif field_property == DynamicFieldProperty.PUSH_ID:
                validate_value = fields.Nested(
                    PushIdSchema, allow_none=True, missing=None, default=None
                )
            elif field_property == DynamicFieldProperty.CARDS:
                validate_value = fields.Nested(
                    CardSchema, allow_none=True, missing=None, default=None
                )
        return Schema.from_dict(
            {
                "changeable": fields.Boolean(
                    default=True, missing=True, allow_none=False
                ),
                "suggest": fields.Boolean(default=True, missing=True, allow_none=False),
                "value": validate_value,
                "status": fields.Int(default=None, allow_none=True),
                "predict": fields.Dict(allow_none=True)
            }
        )

    def validate_merge(self, data, schema_validate_value=None):
        rules = {
            "display_type": fields.Str(
                validate=validate.OneOf([x.value for x in DisplayType])
            ),
            "displayable": fields.Boolean(),
            "editable": fields.Boolean(),
            "field_property": fields.Int(),
            "group": fields.Str(),
            "mergeable": fields.Boolean(),
            "order": fields.Int(),
            "tooltip_i18n": fields.Str(allow_none=True, missing=None, default=None),
            "translate_key": fields.Str(allow_none=True, missing=None, default=None),
        }
        if not schema_validate_value:
            display_type = data.get("display_type")
            field_property = data.get("field_property")
            if display_type in DISPLAY_TYPE_IS_LIST_TYPE:
                rules["field_value"] = fields.List(
                    fields.Nested(
                        self.__validate_field_value__(field_property=field_property)
                    )
                )
            else:
                rules["field_value"] = fields.Nested(
                    self.__validate_field_value__(field_property=field_property)
                )
        else:
            rules["field_value"] = schema_validate_value
        generated_schema = Schema.from_dict(rules)
        return generated_schema().load(data)

    def get_updated_data(self, data):
        if not data:
            data = dict()
        if data.get(ProfileHistoryChangeType.REMOVE):
            old_value = data.get(ProfileHistoryChangeType.REMOVE)[0]
        else:
            old_value = (
                data.get(ProfileHistoryChangeType.CHANGE)[0].get("from")
                if data.get(ProfileHistoryChangeType.CHANGE)
                else None
            )
        if data.get(ProfileHistoryChangeType.ADD):
            new_value = data.get(ProfileHistoryChangeType.ADD)[0]
        else:
            new_value = (
                data.get(ProfileHistoryChangeType.CHANGE)[0].get("to")
                if data.get(ProfileHistoryChangeType.CHANGE)
                else None
            )
        return old_value, new_value

    def get_add_data(self, data):
        if not data:
            data = dict()
        return data.get(ProfileHistoryChangeType.ADD) or []

    def get_remove_data(self, data):
        if not data:
            data = dict()
        return data.get(ProfileHistoryChangeType.REMOVE) or []



def df_get_job_data():
    lst = []
    job_data = [
        {
            "id": 1,
            "name": "Làm ruộng",
            "display_in_form": True
        },
        {
            "id": 2,
            "name": "Bác sỹ",
            "display_in_form": True
        },
        {
            "id": 3,
            "name": "Bán hàng ăn uống",
            "display_in_form": True
        },
        {
            "id": 4,
            "name": "Bán hàng giải khát",
            "display_in_form": True
        },
        {
            "id": 5,
            "name": "Bất động sản",
            "display_in_form": True
        },
        {
            "id": 6,
            "name": "Bộ đội",
            "display_in_form": True
        },
        {
            "id": 7,
            "name": "Bộ đội xuất ngũ",
            "display_in_form": True
        },
        {
            "id": 8,
            "name": "Bốc vác",
            "display_in_form": True
        },
        {
            "id": 9,
            "name": "Buôn bán",
            "display_in_form": True
        },
        {
            "id": 10,
            "name": "Ca sỹ",
            "display_in_form": True
        },
        {
            "id": 11,
            "name": "Cán bộ",
            "display_in_form": True
        },
        {
            "id": 12,
            "name": "Cắt tóc",
            "display_in_form": True
        },
        {
            "id": 13,
            "name": "Cầu đường",
            "display_in_form": True
        },
        {
            "id": 14,
            "name": "Chăn nuôi",
            "display_in_form": True
        },
        {
            "id": 15,
            "name": "Chế biến nông sản",
            "display_in_form": True
        },
        {
            "id": 16,
            "name": "Chủ nhiệm HTX",
            "display_in_form": True
        },
        {
            "id": 17,
            "name": "Công an",
            "display_in_form": True
        },
        {
            "id": 18,
            "name": "Công nhân",
            "display_in_form": True
        },
        {
            "id": 19,
            "name": "Cơ khí",
            "display_in_form": True
        },
        {
            "id": 20,
            "name": "Cơ quan dân chính Đảng",
            "display_in_form": True
        },
        {
            "id": 21,
            "name": "Dạy học",
            "display_in_form": True
        },
        {
            "id": 22,
            "name": "Đánh bắt thuỷ sản",
            "display_in_form": True
        },
        {
            "id": 23,
            "name": "Điện toán ngân hàng",
            "display_in_form": True
        },
        {
            "id": 24,
            "name": "Điều dưỡng",
            "display_in_form": True
        },
        {
            "id": 25,
            "name": "Giám đốc",
            "display_in_form": True
        },
        {
            "id": 26,
            "name": "Giáo viên",
            "display_in_form": True
        },
        {
            "id": 27,
            "name": "Hải quan",
            "display_in_form": True
        },
        {
            "id": 28,
            "name": "Hành nghề y",
            "display_in_form": True
        },
        {
            "id": 29,
            "name": "Học sinh",
            "display_in_form": True
        },
        {
            "id": 30,
            "name": "Học sinh phổ thông",
            "display_in_form": True
        },
        {
            "id": 31,
            "name": "Học sinh trung cấp KTD Nghề",
            "display_in_form": True
        },
        {
            "id": 32,
            "name": "Học Viên",
            "display_in_form": True
        },
        {
            "id": 33,
            "name": "Hội hoạ",
            "display_in_form": True
        },
        {
            "id": 34,
            "name": "IT",
            "display_in_form": True
        },
        {
            "id": 35,
            "name": "Kế hoạch vật tư",
            "display_in_form": True
        },
        {
            "id": 36,
            "name": "Kế toán",
            "display_in_form": True
        },
        {
            "id": 37,
            "name": "Kiểm ngân",
            "display_in_form": True
        },
        {
            "id": 38,
            "name": "Kiểm sát viên",
            "display_in_form": True
        },
        {
            "id": 39,
            "name": "Kiến trúc",
            "display_in_form": True
        },
        {
            "id": 40,
            "name": "Kinh doanh",
            "display_in_form": True
        },
        {
            "id": 41,
            "name": "Kinh doanh Karaoke",
            "display_in_form": True
        },
        {
            "id": 42,
            "name": "Kỹ Sư",
            "display_in_form": True
        },
        {
            "id": 43,
            "name": "Kỹ thuật điện",
            "display_in_form": True
        },
        {
            "id": 44,
            "name": "Lái tàu hoả",
            "display_in_form": True
        },
        {
            "id": 45,
            "name": "Lái xe",
            "display_in_form": True
        },
        {
            "id": 46,
            "name": "Lao động tự do",
            "display_in_form": True
        },
        {
            "id": 47,
            "name": "Lâm nghiệp",
            "display_in_form": True
        },
        {
            "id": 48,
            "name": "Lập trình",
            "display_in_form": True
        },
        {
            "id": 49,
            "name": "Linh mục",
            "display_in_form": True
        },
        {
            "id": 50,
            "name": "Luật sư",
            "display_in_form": True
        },
        {
            "id": 51,
            "name": "Marketing",
            "display_in_form": True
        },
        {
            "id": 52,
            "name": "May mặc",
            "display_in_form": True
        },
        {
            "id": 53,
            "name": "Nha Sỹ",
            "display_in_form": True
        },
        {
            "id": 54,
            "name": "Nhà Báo",
            "display_in_form": True
        },
        {
            "id": 55,
            "name": "Ngân hàng",
            "display_in_form": True
        },
        {
            "id": 56,
            "name": "Nghỉ chế độ 176",
            "display_in_form": True
        },
        {
            "id": 57,
            "name": "Nghỉ hưu",
            "display_in_form": True
        },
        {
            "id": 58,
            "name": "Nhân viên",
            "display_in_form": True
        },
        {
            "id": 59,
            "name": "Nhân viên bảo vệ",
            "display_in_form": True
        },
        {
            "id": 60,
            "name": "Nhân viên nhà hàng",
            "display_in_form": True
        },
        {
            "id": 61,
            "name": "Nội trợ",
            "display_in_form": True
        },
        {
            "id": 62,
            "name": "P/Giám đốc",
            "display_in_form": True
        },
        {
            "id": 63,
            "name": "Phó Tổng giám đốc",
            "display_in_form": True
        },
        {
            "id": 64,
            "name": "Phụ xe ô tô",
            "display_in_form": True
        },
        {
            "id": 65,
            "name": "Sinh Viên",
            "display_in_form": True
        },
        {
            "id": 66,
            "name": "Sinh viên cao đẳng",
            "display_in_form": True
        },
        {
            "id": 67,
            "name": "Sinh viên đại học",
            "display_in_form": True
        },
        {
            "id": 68,
            "name": "Sơn",
            "display_in_form": True
        },
        {
            "id": 69,
            "name": "Sửa chữa điện tử",
            "display_in_form": True
        },
        {
            "id": 70,
            "name": "Sửa chữa ôtô điện máy",
            "display_in_form": True
        },
        {
            "id": 71,
            "name": "Sửa chữa xe máy",
            "display_in_form": True
        },
        {
            "id": 72,
            "name": "Thợ ảnh",
            "display_in_form": True
        },
        {
            "id": 73,
            "name": "Thợ hàn",
            "display_in_form": True
        },
        {
            "id": 74,
            "name": "Thợ mộc",
            "display_in_form": True
        },
        {
            "id": 75,
            "name": "Thợ sắt",
            "display_in_form": True
        },
        {
            "id": 76,
            "name": "Thợ Sơn",
            "display_in_form": True
        },
        {
            "id": 77,
            "name": "Thợ xây",
            "display_in_form": True
        },
        {
            "id": 78,
            "name": "Thú y",
            "display_in_form": True
        },
        {
            "id": 79,
            "name": "Thủ kho",
            "display_in_form": True
        },
        {
            "id": 80,
            "name": "Thủ quỹ",
            "display_in_form": True
        },
        {
            "id": 81,
            "name": "Thuế vụ",
            "display_in_form": True
        },
        {
            "id": 82,
            "name": "Thuỷ lợi",
            "display_in_form": True
        },
        {
            "id": 83,
            "name": "Thuỷ thủ tàu biển",
            "display_in_form": True
        },
        {
            "id": 84,
            "name": "Tổng Giám đốc",
            "display_in_form": True
        },
        {
            "id": 85,
            "name": "Tu sỹ",
            "display_in_form": True
        },
        {
            "id": 86,
            "name": "Văn nghệ sỹ",
            "display_in_form": True
        },
        {
            "id": 87,
            "name": "Vệ sinh môi trường",
            "display_in_form": True
        },
        {
            "id": 88,
            "name": "Viễn thông",
            "display_in_form": True
        },
        {
            "id": 89,
            "name": "Xe ôm",
            "display_in_form": True
        },
        {
            "id": 90,
            "name": "Xích lô",
            "display_in_form": True
        },
        {
            "id": 91,
            "name": "Y tá",
            "display_in_form": True
        },
        {
            "id": 92,
            "name": "Khác",
            "display_in_form": True
        },
        {
            "id": 93,
            "name": "Giảng viên",
            "display_in_form": True
        }
    ]
    for job in job_data:
        job["display_in_form"] = True
        lst.append(job)
    return lst



from marshmallow import fields, validate, Schema

from mobio.libs.profiling_mf.common_helper import CommonHelper
from mobio.libs.profiling_mf.merge_fields.base_merge import BaseMerge, MergeListGroup
from mobio.libs.profiling_mf.profiling_common import DisplayType, ProfileHistoryChangeType
from mobio.libs.profiling_mf.profiling_data.hobby_data import df_get_hobby_data
from mobio.libs.profiling_mf.profiling_schema import HobbySchema


class MergeHobby(BaseMerge):
    def serialize_data(
        self,
        suggest_data,
        profile_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
        predict=None
    ):
        lst_suggest = []
        profile_data = profile_data if profile_data is not None else []
        for a in profile_data:
            lst_suggest.append(self.__build_value__(value=a, suggest=True))
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=True,
            editable=True,
            mergeable=True,
            order=1,
            group=MergeListGroup.ACTIVITY,
            value=lst_suggest,
        )

    def set_filter_value(self, suggest_filter_data, profile_data):
        pass

    def serialize_origin_data(
        self,
        suggest_data,
        origin_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
    ):
        lst_suggest = []
        profile_data = origin_data if origin_data is not None else []
        if isinstance(profile_data, str):
            try:
                arr_ids = list(map(lambda x: int(x), profile_data.split(";")))
            except Exception as ex:
                print("MERGE::HOBBY = {}".format(ex))
                arr_ids = []
        elif isinstance(profile_data, list):
            try:
                arr_ids = list(map(lambda x: int(x), profile_data))
            except Exception as ex:
                print("MERGE::HOBBY = {}".format(ex))
                arr_ids = []
        else:
            print("ERROR HOBBY: {} invalid".format(profile_data))
            arr_ids = []
        hobby_data = df_get_hobby_data()
        arr_hobby = list(filter(lambda x: x["id"] in arr_ids, hobby_data))
        arr_hobby = list(
            map(
                lambda x: CommonHelper.create_simple_data_type(
                    _id=x["id"], _name=x["name"]
                ),
                arr_hobby,
            )
        )
        for a in arr_hobby:
            lst_suggest.append(self.__build_value__(value=a, suggest=True))
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=True,
            editable=True,
            mergeable=True,
            order=1,
            group=MergeListGroup.ACTIVITY,
            value=lst_suggest,
        )

    def merge_data(self, target_data, source_data, field_key, is_master_data=False):
        target_data[field_key] = target_data.get(field_key) if target_data.get(field_key) else []
        if source_data:
            hobby_data = df_get_hobby_data()
            set_hobby = set([x.get("id") for x in target_data.get(field_key)]) if not is_master_data else set()
            set_hobby.update(
                [x.get("value").get("id") for x in source_data.get("field_value")]
            )
            arr_ids = list(set_hobby)
            arr_hobby = list(filter(lambda x: x["id"] in arr_ids, hobby_data))
            arr_hobby = list(
                map(
                    lambda x: CommonHelper.create_simple_data_type(
                        _id=x["id"], _name=x["name"]
                    ),
                    arr_hobby,
                )
            )
            target_data[field_key] = arr_hobby

    def validate_merge(self, data, schema_validate_value=None):
        schema_validate_value = {
            "changeable": fields.Boolean(
                default=True, missing=True, allow_none=False
            ),
            "suggest": fields.Boolean(default=True, missing=True, allow_none=False),
            "value": fields.Nested(HobbySchema),
            "status": fields.Int(default=None, allow_none=True),
            "predict": fields.Dict(allow_none=True)
        }
        rules = {
            "display_type": fields.Str(
                validate=validate.OneOf([x.value for x in DisplayType])
            ),
            "displayable": fields.Boolean(),
            "editable": fields.Boolean(),
            "field_property": fields.Int(),
            "group": fields.Str(),
            "mergeable": fields.Boolean(),
            "order": fields.Int(),
            "tooltip_i18n": fields.Str(allow_none=True, missing=None, default=None),
            "translate_key": fields.Str(allow_none=True, missing=None, default=None),
            "field_value": fields.List(fields.Nested(Schema.from_dict(schema_validate_value)))
        }

        generated_schema = Schema.from_dict(rules)
        return generated_schema().load(data)

    def get_add_data(self, data):
        if not data:
            data = dict()
        result = []
        if data.get(ProfileHistoryChangeType.ADD):
            result = [x.get("id") for x in data.get(ProfileHistoryChangeType.ADD)]
        return result

    def get_remove_data(self, data):
        if not data:
            data = dict()
        result = []
        if data.get(ProfileHistoryChangeType.REMOVE):
            result = [x.get("id") for x in data.get(ProfileHistoryChangeType.REMOVE)]
        return result



def df_get_frequently_data():
    lst = []
    frequently_demand_data = [
        {
            "id": 1,
            "name": "Đi du lịch",
            "name_en": "Travel",
            "display_in_form": True
        },
        {
            "id": 2,
            "name": "Mua sắm online",
            "name_en": "Online Shopping",
            "display_in_form": True
        },
        {
            "id": 3,
            "name": "Mua sắm siêu thị",
            "name_en": "Offline Shopping",
            "display_in_form": True
        },
        {
            "id": 4,
            "name": "Ăn uống nhà hàng",
            "name_en": "Restaurant",
            "display_in_form": True
        },
        {
            "id": 5,
            "name": "Giải trí",
            "name_en": "Entertainment",
            "display_in_form": True
        },
        {
            "id": 6,
            "name": "Đóng tiền học phí",
            "name_en": "Pay tuition fee",
            "display_in_form": True
        }
    ]
    for frequently in frequently_demand_data:
        frequently["display_in_form"] = True
        lst.append(frequently)
    return lst



from mobio.libs.profiling_mf.merge_fields.base_merge import BaseMerge


class MergePeopleId(BaseMerge):
    def serialize_data(
        self,
        suggest_data,
        profile_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
        predict=None
    ):
        pass

    def set_filter_value(self, suggest_filter_data, profile_data):
        pass

    def serialize_origin_data(
        self,
        suggest_data,
        origin_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
    ):
        pass

    def merge_data(self, target_data, source_data, field_key, is_master_data=False):
        pass



def df_get_degree_data():
    degree_data = [
        {
            'id': 1,
            'name': 'Sau đại học',
            'display_in_form': True
        },
        {
            'id': 2,
            'name': 'Đại học',
            'display_in_form': True
        },
        {
            'id': 3,
            'name': ' Cao đẳng',
            'display_in_form': True
        },
        {
            'id': 4,
            'name': 'Phổ thông',
            'display_in_form': True
        },
        {
            'id': 5,
            'name': 'Trung cấp',
            'display_in_form': True
        }
    ]
    lst = []
    for degree in degree_data:
        degree["display_in_form"] = True
        lst.append(degree)
    return lst



GENDER_DATA = [
    {
        "id": 2,
        "name": "Nam",
        'display_in_form': True
    },
    {
        "id": 3,
        "name": "Nữ",
        'display_in_form': True
    },
    {
        "id": 1,
        "name": "Không xác định",
        'display_in_form': True
    }
]


def df_get_gender_data():
    lst = []
    for gender in GENDER_DATA:
        lst.append(gender)
    return lst



from mobio.libs.profiling_mf.merge_fields.base_merge import BaseMerge, MergeListGroup


class MergeSocialName(BaseMerge):
    def serialize_data(
        self,
        suggest_data,
        profile_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
        predict=None
    ):
        lst_social = []
        for social in profile_data:
            _id = social.get("id")
            _name = social.get("name")
            _social_id = social.get("social_id")
            unique_suggest_value = "{}:{}".format(_id, _name)
            suggest = True if unique_suggest_value not in set_unique_suggest_values else False
            lst_social.append(
                self.__build_value__(value=social, suggest=suggest, changealbe=False)
            )
            set_unique_suggest_values.add(unique_suggest_value)

        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=False,
            editable=False,
            mergeable=True,
            order=1,
            group=MergeListGroup.INFORMATION,
            value=lst_social,
        )

    def set_filter_value(self, suggest_filter_data: set, profile_data):
        for social in profile_data:
            if social.get("name"):
                suggest_filter_data.add(social.get("name"))

    def serialize_origin_data(
        self,
        suggest_data,
        origin_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
    ):
        pass

    def merge_data(self, target_data, source_data, field_key, is_master_data=False):
        if source_data:
            lst_social_name = target_data.get(field_key, [])
            set_social_name = set()
            for source_social in source_data.get('field_value'):
                _id = source_social.get("id")
                _name = source_social.get("name")
                _social_id = source_social.get("social_id")
                unique_suggest_value = "{}:{}".format(_id, _name)
                if unique_suggest_value not in set_social_name:
                    lst_social_name.append(source_social)
                    set_social_name.add(unique_suggest_value)
            target_data[field_key] = lst_social_name



import uuid

from mobio.libs.profiling_mf.common_helper import CommonHelper
from mobio.libs.profiling_mf.merge_fields.base_merge import BaseMerge, MergeListGroup


class MergeMerchantId(BaseMerge):
    def serialize_data(
        self,
        suggest_data,
        profile_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
        predict=None
    ):
        lst_suggest_merchant_id = []
        for merchant_id in profile_data:
            lst_suggest_merchant_id.append(
                self.__build_value__(
                    value=str(merchant_id), suggest=True, predict=predict
                )
            )
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=False,
            editable=False,
            mergeable=True,
            order=1,
            group=MergeListGroup.INFORMATION,
            value=lst_suggest_merchant_id,
        )

    def set_filter_value(self, suggest_filter_data, profile_data):
        if type(profile_data) == list:
            for merchant_id in profile_data:
                if merchant_id:
                    suggest_filter_data.add(str(merchant_id))
        elif type(profile_data) == str:
            suggest_filter_data.add(profile_data)

    def serialize_origin_data(
        self,
        suggest_data,
        origin_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
    ):
        lst_merchant_id = []
        if type(origin_data) == list:
            lst_merchant_id = [str(x) for x in origin_data if x]
        elif type(origin_data) in [str, uuid.UUID]:
            lst_merchant_id = [str(origin_data)]
        lst_suggest_merchant_id = []
        for merchant_id in lst_merchant_id:
            lst_suggest_merchant_id.append(
                self.__build_value__(
                    value=str(merchant_id), suggest=True,
                )
            )
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=False,
            editable=False,
            mergeable=True,
            order=1,
            group=MergeListGroup.INFORMATION,
            value=lst_suggest_merchant_id,
        )

    def merge_data(self, target_data, source_data, field_key, is_master_data=False):
        target_data[field_key] = target_data.get(field_key) if target_data.get(field_key) else []
        if source_data:
            for row in source_data.get("field_value"):
                suggest = row.get("suggest")
                value = CommonHelper.normalize_uuid(row.get("value"))
                if (
                    suggest
                    and value is not None
                    and value not in target_data.get(field_key)
                ):
                    target_data[field_key].append(value)



from marshmallow import fields, Schema, ValidationError, validate
from mobio.libs.profiling_mf.merge_fields.base_merge import BaseMerge, MergeListGroup
from mobio.libs.profiling_mf.profiling_common import DisplayType
from mobio.libs.profiling_mf.profiling_schema import DeviceSchema


class MergeDevices(BaseMerge):
    def serialize_data(
        self,
        suggest_data,
        profile_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
        predict=None,
    ):
        lst_suggest_device = []
        set_device_id = set()
        profile_data = profile_data if profile_data else []
        for device in profile_data:
            if device.get("device_id") not in set_device_id:
                set_device_id.add(device.get("device_id"))
                lst_suggest_device.append(
                    self.__build_value__(value=device, suggest=True)
                )
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=False,
            editable=False,
            mergeable=True,
            order=1,
            group=MergeListGroup.OTHER,
            value=lst_suggest_device,
        )

    def set_filter_value(self, suggest_filter_data, profile_data):
        pass

    def serialize_origin_data(
        self,
        suggest_data,
        origin_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
    ):
        lst_suggest_device = []
        if type(origin_data) == list:
            set_device_id = set()
            for device in origin_data:
                if device.get("device_id") not in set_device_id:
                    set_device_id.add(device.get("device_id"))
                    lst_suggest_device.append(
                        self.__build_value__(value=device, suggest=True)
                    )
        elif type(origin_data) == dict:
            lst_suggest_device.append(
                self.__build_value__(value=origin_data, suggest=True)
            )
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=False,
            editable=False,
            mergeable=True,
            order=1,
            group=MergeListGroup.OTHER,
            value=lst_suggest_device,
        )

    def merge_data(self, target_data, source_data, field_key, is_master_data=False):
        if source_data:
            target_data[field_key] = (
                target_data.get(field_key) if target_data.get(field_key) else []
            )
            set_device_id = set(
                [x.get("device_id") for x in target_data.get(field_key)]
            )
            for device_data in source_data.get("field_value"):
                device = device_data.get("value")
                if device.get("device_id") not in set_device_id:
                    target_data[field_key].append(device)
                    set_device_id.add(device.get("device_id"))

    def validate_merge(self, data, schema_validate_value=None):
        schema_validate_value = {
            "changeable": fields.Boolean(default=True, missing=True, allow_none=False),
            "suggest": fields.Boolean(default=True, missing=True, allow_none=False),
            "value": fields.Nested(DeviceSchema),
            "status": fields.Int(default=None, allow_none=True),
            "predict": fields.Dict(allow_none=True),
        }
        field_value = data.get("field_value", [])
        tmp_field_value = []
        for value in field_value:
            generated_schema_value = Schema.from_dict(schema_validate_value)
            try:
                valid_value = generated_schema_value().load(value)
                tmp_field_value.append(valid_value)
            except ValidationError as ex:
                print(
                    "MergeDevices:: validation value error: {}".format(ex)
                )
        data["field_value"] = tmp_field_value
        rules = {
            "display_type": fields.Str(
                validate=validate.OneOf([x.value for x in DisplayType])
            ),
            "displayable": fields.Boolean(),
            "editable": fields.Boolean(),
            "field_property": fields.Int(),
            "group": fields.Str(),
            "mergeable": fields.Boolean(),
            "order": fields.Int(),
            "tooltip_i18n": fields.Str(allow_none=True, missing=None, default=None),
            "translate_key": fields.Str(allow_none=True, missing=None, default=None),
            "field_value": fields.List(
                fields.Nested(Schema.from_dict(schema_validate_value))
            ),
        }

        generated_schema = Schema.from_dict(rules)
        return generated_schema().load(data)



from mobio.libs.profiling_mf.merge_fields.base_merge import BaseMerge, MergeListGroup


class MergeCards(BaseMerge):
    def serialize_data(
        self,
        suggest_data,
        profile_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
        predict=None
    ):
        lst_cards = list()
        profile_data = profile_data if profile_data is not None else []
        for card in profile_data:
            lst_cards.append(
                self.__build_value__(value=card, suggest=True, predict=predict),
            )
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=False,
            editable=False,
            mergeable=False,
            order=1,
            group=MergeListGroup.LOYALTY,
            value=lst_cards,
        )

    def set_filter_value(self, suggest_filter_data, profile_data):
        pass

    def serialize_origin_data(
        self,
        suggest_data,
        origin_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
    ):
        origin_data = []
        lst_cards = []
        for card in origin_data:
            lst_cards.append(
                self.__build_value__(value=card, suggest=True),
            )
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=False,
            editable=False,
            mergeable=False,
            order=1,
            group=MergeListGroup.LOYALTY,
            value=lst_cards,
        )

    def merge_data(self, target_data, source_data, field_key, is_master_data=False):
        target_data[field_key] = target_data.get(field_key) if target_data.get(field_key) else []
        set_data = set([x.get("card_id") for x in target_data.get(field_key)])
        if source_data:
            for data in source_data.get("field_value"):
                suggest = data.get("suggest")
                value = data.get("value")
                if (
                    suggest
                    and value is not None
                    and value.get("card_id") not in set_data
                ):
                    set_data.add(value.strip())
                    target_data[field_key].append(value)



from marshmallow import fields, validate, Schema
from mobio.libs.profiling_mf import ProfileStructure, ProfileTagsStructure, TagInteractiveStructure
from mobio.libs.profiling_mf.common_helper import CommonHelper
from mobio.libs.profiling_mf.merge_fields.base_merge import BaseMerge, MergeListGroup
from mobio.libs.profiling_mf.profiling_common import DisplayType, ProfileHistoryChangeType
from mobio.libs.profiling_mf.profiling_schema import ProfileTagSchema


class MergeProfileTags(BaseMerge):
    def serialize_data(
        self,
        suggest_data,
        profile_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
        predict=None
    ):
        set_data = set()
        lst_suggest_data = []
        profile_data = profile_data if profile_data is not None else []
        for a in profile_data:
            if type(a) == dict and a.get(ProfileTagsStructure.ID) not in set_data:
                lst_suggest_data.append(self.__build_value__(value=a, suggest=True))
                set_data.add(a.get(ProfileTagsStructure.ID))
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=False,
            editable=False,
            mergeable=True,
            order=1,
            group=MergeListGroup.ACTIVITY,
            value=lst_suggest_data,
        )

    def set_filter_value(self, suggest_filter_data, profile_data):
        profile_data = profile_data if profile_data is not None else []
        for tag in profile_data:
            if tag and type(tag) == dict:
                suggest_filter_data.add(tag.get(ProfileTagsStructure.TAG))

    def serialize_origin_data(
        self,
        suggest_data,
        origin_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
    ):
        set_data = set()
        lst_suggest_data = []
        origin_data = origin_data if origin_data is not None else []
        if type(origin_data) == list:
            for a in origin_data:
                if type(a) == dict and a.get(ProfileTagsStructure.ID) not in set_data:
                    lst_suggest_data.append(self.__build_value__(value=a, suggest=True))
                    set_data.add(a.get(ProfileTagsStructure.ID))
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=False,
            editable=False,
            mergeable=True,
            order=1,
            group=MergeListGroup.ACTIVITY,
            value=lst_suggest_data,
        )

    def merge_data(self, target_data, source_data, field_key, is_master_data=False):
        target_data[field_key] = (
            target_data.get(field_key) if target_data.get(field_key) else []
        )
        target_data[ProfileStructure.TAGS_INTERACTIVE] = (
            target_data.get(ProfileStructure.TAGS_INTERACTIVE)
            if target_data.get(ProfileStructure.TAGS_INTERACTIVE)
            else []
        )
        if source_data:
            for data_suggest in source_data.get("field_value"):
                suggest = data_suggest.get("suggest")
                value = data_suggest.get("value")
                if suggest and value is not None:
                    exists_tags = next(
                        (
                            x
                            for x in target_data.get(field_key)
                            if x.get(ProfileTagsStructure.MERCHANT_ID)
                            == value.get(ProfileTagsStructure.MERCHANT_ID)
                            and x.get(ProfileTagsStructure.ID) == value.get(ProfileTagsStructure.ID)
                        ),
                        None,
                    )
                    if not exists_tags:
                        target_data[field_key].append(value)
        for tag in target_data.get(field_key):
            if tag.get(ProfileTagsStructure.ID) not in target_data.get(
                ProfileStructure.TAGS_INTERACTIVE
            ):
                tag_interactive = CommonHelper.create_tag_interactive()
                tag_interactive[TagInteractiveStructure.TAG_ID] = tag.get(ProfileTagsStructure.ID)
                target_data[ProfileStructure.TAGS_INTERACTIVE].append(tag_interactive)

    def validate_merge(self, data, schema_validate_value=None):
        schema_validate_value = {
            "changeable": fields.Boolean(default=True, missing=True, allow_none=False),
            "suggest": fields.Boolean(default=True, missing=True, allow_none=False),
            "value": fields.Nested(ProfileTagSchema),
            "status": fields.Int(default=None, allow_none=True),
            "predict": fields.Dict(allow_none=True)
        }
        rules = {
            "display_type": fields.Str(
                validate=validate.OneOf([x.value for x in DisplayType])
            ),
            "displayable": fields.Boolean(),
            "editable": fields.Boolean(),
            "field_property": fields.Int(),
            "group": fields.Str(),
            "mergeable": fields.Boolean(),
            "order": fields.Int(),
            "tooltip_i18n": fields.Str(allow_none=True, missing=None, default=None),
            "translate_key": fields.Str(allow_none=True, missing=None, default=None),
            "field_value": fields.List(
                fields.Nested(Schema.from_dict(schema_validate_value))
            ),
        }

        generated_schema = Schema.from_dict(rules)
        return generated_schema().load(data)

    def get_add_data(self, data):
        if not data:
            data = dict()
        if ProfileHistoryChangeType.ADD not in data:
            data[ProfileHistoryChangeType.ADD] = []
        return [x.get(ProfileTagsStructure.ID) for x in data.get(ProfileHistoryChangeType.ADD)]

    def get_remove_data(self, data):
        if not data:
            data = dict()
        if ProfileHistoryChangeType.REMOVE not in data:
            data[ProfileHistoryChangeType.REMOVE] = []
        return [x.get(ProfileTagsStructure.ID) for x in data.get(ProfileHistoryChangeType.REMOVE)]



def df_get_religiousness_data():
    lst = []
    religiousness_data = [
        {
            "id": 1,
            "name": "Lương giáo",
            'display_in_form': True
        },
        {
            "id": 2,
            "name": "Phật giáo",
            'display_in_form': True
        },
        {
            "id": 3,
            "name": "Công giáo",
            'display_in_form': True
        },
        {
            "id": 4,
            "name": "Cao Đài",
            'display_in_form': True
        },
        {
            "id": 5,
            "name": "Hòa Hảo",
            'display_in_form': True
        },
        {
            "id": 6,
            "name": "Tin Lành",
            'display_in_form': True
        },
        {
            "id": 7,
            "name": "Hồi Giáo",
            'display_in_form': True
        },
        {
            "id": 8,
            "name": "Bà La Môn",
            'display_in_form': True
        },
        {
            "id": 9,
            "name": "Đạo Tứ ấn hiếu nghĩa",
            'display_in_form': True
        },
        {
            "id": 10,
            "name": "Tịnh độ cư sĩ Phật hội Việt Nam",
            'display_in_form': True
        },
        {
            "id": 11,
            "name": "Bửu sơn Kỳ hương",
            'display_in_form': True
        },
        {
            "id": 12,
            "name": "Minh Sư Đạo",
            'display_in_form': True
        },
        {
            "id": 13,
            "name": "Bahá'í",
            'display_in_form': True
        },
        {
            "id": 14,
            "name": "Minh Lý Đạo",
            'display_in_form': True
        }
    ]
    for religiousness in religiousness_data:
        religiousness['display_in_form'] = True
        lst.append(religiousness)
    return lst



from mobio.libs.profiling_mf.merge_fields.base_merge import BaseMerge, MergeListGroup


class MergeVibCards(BaseMerge):
    def serialize_data(
        self,
        suggest_data,
        profile_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
        predict=None
    ):
        lst_suggest = []
        set_id = set()
        for data in profile_data:
            if data.get("card_id") not in set_id:
                set_id.add(data.get("card_id"))
                lst_suggest.append(
                    self.__build_value__(value=data, suggest=True)
                )
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=False,
            editable=False,
            mergeable=True,
            order=1,
            group=MergeListGroup.DYNAMIC,
            value=lst_suggest,
        )

    def set_filter_value(self, suggest_filter_data, profile_data):
        pass

    def serialize_origin_data(
        self,
        suggest_data,
        origin_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
    ):
        lst_suggest = []
        set_id = set()
        for data in origin_data:
            if data.get("card_id") not in set_id:
                set_id.add(data.get("card_id"))
                lst_suggest.append(
                    self.__build_value__(value=data, suggest=True)
                )
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=False,
            editable=False,
            mergeable=True,
            order=1,
            group=MergeListGroup.DYNAMIC,
            value=lst_suggest,
        )

    def merge_data(self, target_data, source_data, field_key, is_master_data=False):
        if source_data:
            target_data[field_key] = target_data.get(target_data, [])
            set_id = set([x.get("card_id") for x in target_data.get(target_data)])
            for data in source_data.get('field_value'):
                if data.get("card_id") not in set_id:
                    target_data[field_key].append(data)
                    set_id.add(data.get("card_id"))



from mobio.libs.profiling_mf.merge_fields.base_merge import BaseMerge, MergeListGroup
from mobio.libs.profiling_mf.profiling_common import DISPLAY_TYPE_IS_LIST_TYPE


class MergeDynamic(BaseMerge):
    def serialize_data(
        self,
        suggest_data,
        profile_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
        predict=None,
    ):
        if display_type in DISPLAY_TYPE_IS_LIST_TYPE:
            field_value = []
            if profile_data:
                for row in profile_data:
                    field_value.append(self.__build_value__(value=row, suggest=True))
        else:
            suggest = (
                True
                if profile_data is not None
                and profile_data != ""
                and field_key not in set_suggest_fields
                else False
            )
            field_value = self.__build_value__(value=profile_data, suggest=suggest)
            if suggest:
                set_suggest_fields.add(field_key)
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=True,
            editable=True,
            mergeable=True,
            order=1,
            group=MergeListGroup.DYNAMIC,
            value=field_value,
        )

    def set_filter_value(self, suggest_filter_data, profile_data):
        pass

    def serialize_origin_data(
        self,
        suggest_data,
        origin_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
    ):
        if display_type in DISPLAY_TYPE_IS_LIST_TYPE:
            field_value = []
            if origin_data:
                for row in origin_data:
                    field_value.append(self.__build_value__(value=row, suggest=True))
        else:
            suggest = (
                True
                if origin_data is not None
                and origin_data != ""
                and field_key not in set_suggest_fields
                else False
            )
            field_value = self.__build_value__(value=origin_data, suggest=suggest)
            if suggest:
                set_suggest_fields.add(field_key)
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=True,
            editable=True,
            mergeable=True,
            order=1,
            group=MergeListGroup.DYNAMIC,
            value=field_value,
        )

    def merge_data(self, target_data, source_data, field_key, is_master_data=False):
        if source_data:
            if type(source_data.get("field_value")) == list:
                target_data[field_key] = (
                    target_data.get(field_key) if target_data.get(field_key) else []
                )
                set_data = (
                    set(target_data.get(field_key)) if not is_master_data else set()
                )
                for row in source_data.get("field_value"):
                    suggest = row.get("suggest")
                    value = row.get("value")
                    if suggest and value is not None:
                        set_data.add(value)
                target_data[field_key] = list(set_data)
            else:
                suggest = source_data.get("field_value").get("suggest")
                value = source_data.get("field_value").get("value")
                if suggest and value is not None:
                    target_data[field_key] = value


if __name__ == "__main__":
    old, new = MergeDynamic().get_updated_data(data={
                'display_type': 'date_picker',
                'key': '_dyn_truong_test_datepicker_ddmm_1616574483327',
                'field_key': '_dyn_truong_test_datepicker_ddmm_1616574483327',
                'translate_key': '',
                'add': [],
                'format': 'dd/mm',
                'change': [
                    {
                        'from': '2099-09-11T00:00:00.000000Z',
                        'to': '2099-08-09T00:00:00.000000Z'
                    }
                ],
                'field_property': 3,
                'field_name': 'Truong_test_datepicker_DDMM',
                'remove': []
            })
    print('old: {}, new: {}'.format(old, new))



def df_get_marital_status_data():
    lst = []
    marital_status_data = [
        {
            "id": 1,
            "name": "Độc thân",
            'display_in_form': True
        },
        {
            "id": 2,
            "name": "Đã đính hôn",
            'display_in_form': True
        },
        {
            "id": 3,
            "name": "Đã kết hôn",
            'display_in_form': True
        },
        {
            "id": 4,
            "name": "Ly thân",
            'display_in_form': True
        },
        {
            "id": 5,
            "name": "Đã ly hôn",
            'display_in_form': True
        },
        {
            "id": 6,
            "name": "Quả phụ",
            'display_in_form': True
        },
        {
            "id": 7,
            "name": "Góa vợ",
            'display_in_form': True
        }
    ]
    for marital in marital_status_data:
        marital["display_in_form"] = True
        lst.append(marital)
    return lst



import uuid
import re
from dateutil.parser import parse
import pytz
from datetime import datetime
import phonenumbers
from mobio.libs.profiling_mf import TagInteractiveStructure
from mobio.libs.profiling_mf.profiling_common import PhoneVerifyStatus, EmailValidateStatus


class CommonHelper:
    @staticmethod
    def normalize_uuid(some_uuid):
        if isinstance(some_uuid, str):
            return uuid.UUID(some_uuid)
        return some_uuid

    @staticmethod
    def create_tag_interactive():
        tag_interactive = dict(
            {
                TagInteractiveStructure.TAG_ID: None,
                TagInteractiveStructure.INTERACTIVE_TOTAL: 0,
                TagInteractiveStructure.INTERACTIVE_3_DAYS: 0,
                TagInteractiveStructure.INTERACTIVE_7_DAYS: 0,
                TagInteractiveStructure.INTERACTIVE_30_DAYS: 0,
                TagInteractiveStructure.LAST_ACTION_TIME: None,
            }
        )
        return tag_interactive

    @staticmethod
    def create_simple_data_type(_id=None, _name=None):
        return {"id": _id, "name": _name}

    @staticmethod
    def parse_time(dt):
        tz_str = """-12 Y
            -11 X NUT SST
            -10 W CKT HAST HST TAHT TKT
            -9 V AKST GAMT GIT HADT HNY
            -8 U AKDT CIST HAY HNP PST PT
            -7 T HAP HNR MST PDT
            -6 S CST EAST GALT HAR HNC MDT
            -5 R CDT COT EASST ECT EST ET HAC HNE PET
            -4 Q AST BOT CLT COST EDT FKT GYT HAE HNA PYT
            -3 P ADT ART BRT CLST FKST GFT HAA PMST PYST SRT UYT WGT
            -2 O BRST FNT PMDT UYST WGST
            -1 N AZOT CVT EGT
            0 Z EGST GMT UTC WET WT
            1 A CET DFT WAT WEDT WEST
            2 B CAT CEDT CEST EET SAST WAST
            3 C EAT EEDT EEST IDT MSK
            4 D AMT AZT GET GST KUYT MSD MUT RET SAMT SCT
            5 E AMST AQTT AZST HMT MAWT MVT PKT TFT TJT TMT UZT YEKT
            6 F ALMT BIOT BTT IOT KGT NOVT OMST YEKST
            7 G CXT DAVT HOVT ICT KRAT NOVST OMSST THA WIB
            8 H ACT AWST BDT BNT CAST HKT IRKT KRAST MYT PHT SGT ULAT WITA WST
            9 I AWDT IRKST JST KST PWT TLT WDT WIT YAKT
            10 K AEST ChST PGT VLAT YAKST YAPT
            11 L AEDT LHDT MAGT NCT PONT SBT VLAST VUT
            12 M ANAST ANAT FJT GILT MAGST MHT NZST PETST PETT TVT WFT
            13 FJST NZDT
            11.5 NFT
            10.5 ACDT LHST
            9.5 ACST
            6.5 CCT MMT
            5.75 NPT
            5.5 SLT
            4.5 AFT IRDT
            3.5 IRST
            -2.5 HAT NDT
            -3.5 HNT NST NT
            -4.5 HLV VET
            -9.5 MART MIT"""
        tzd = {}
        for tz_descr in map(str.split, tz_str.split("\n")):
            tz_offset = int(float(tz_descr[0]) * 3600)
            for tz_code in tz_descr[1:]:
                tzd[tz_code] = tz_offset
        try:
            if isinstance(dt, datetime):
                result = dt
            else:
                result = parse(dt, tzinfos=tzd)
            return (
                result.astimezone(pytz.utc).replace(tzinfo=None)
                if result.tzinfo
                else result
            )
        except Exception as ex:
            print("UserHelper:: parse_time error: {}".format(ex))
            return None

    @staticmethod
    def validate_email(email):
        if not email:
            return False
        try:
            valid = re.match("(^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$)", email)
        except Exception as ex:
            print('validate_email:ERROR: %s' % ex)
            valid = None
        return True if valid is not None else False

    def chuan_hoa_so_dien_thoai_theo_dau_so_moi(self, so_dien_thoai):
        so_dien_thoai = re.sub(r'^(0|\+84|84)12([068])', r'\g<1>7\g<2>', so_dien_thoai)
        so_dien_thoai = re.sub("^(0|\+84|84)121", "\g<1>79", so_dien_thoai)
        so_dien_thoai = re.sub("^(0|\+84|84)122", "\g<1>77", so_dien_thoai)
        so_dien_thoai = re.sub("^(0|\+84|84)12([345])", "\g<1>8\g<2>", so_dien_thoai)
        so_dien_thoai = re.sub("^(0|\+84|84)127", "\g<1>81", so_dien_thoai)
        so_dien_thoai = re.sub("^(0|\+84|84)129", "\g<1>82", so_dien_thoai)
        so_dien_thoai = re.sub("^(0|\+84|84)16([2-9])", "\g<1>3\g<2>", so_dien_thoai)
        so_dien_thoai = re.sub("^(0|\+84|84)18([68])", "\g<1>5\g<2>", so_dien_thoai)
        so_dien_thoai = re.sub("^(0|\+84|84)199", "\g<1>59", so_dien_thoai)

        return so_dien_thoai

    def validate_phone(self, phone_number):
        try:
            pattern_phone_number = "^(\+?84|0)?(1(2([0-9])|6([2-9])|88|86|99)|9([0-9]{1})|8([0-9]{1})|7[0|6-9]|3[2-9]|5[2|5|6|8|9])+([0-9]{7})$"
            valid = re.match(pattern_phone_number, phone_number)
        except Exception as ex:
            print('validate_phone:ERROR: %s' % ex)
            valid = None
        return True if valid is not None else False

    def chuan_hoa_so_dien_thoai_v2(self, so_dien_thoai):
        if not so_dien_thoai:
            return None
        so_dien_thoai = self.chuan_hoa_so_dien_thoai_theo_dau_so_moi(so_dien_thoai)
        try:
            parse = phonenumbers.parse(so_dien_thoai, 'VN')
            if so_dien_thoai.startswith('+84') or so_dien_thoai.startswith('84'):
                is_valid = self.validate_phone(so_dien_thoai)
            else:
                is_valid = phonenumbers.is_valid_number(parse)
            result = phonenumbers.format_number(parse, phonenumbers.PhoneNumberFormat.E164)
        except Exception as e:
            print("chuan_hoa_so_dien_thoai_v2:: Exception: {},{} ".format(so_dien_thoai, str(e)))
            is_valid = False
            result = None
        if is_valid:
            return result
        else:
            return None

    @staticmethod
    def set_primary_email_from_secondary(profile):
        if len(profile.get('secondary_emails').get('secondary')) > 0:
            lst_secondary = list(profile.get('secondary_emails').get('secondary'))
            profile['primary_email'] = lst_secondary.pop(0)
            profile['secondary_emails']['secondary_size'] = len(lst_secondary)
            profile['secondary_emails']['secondary'] = lst_secondary
        return profile

    @staticmethod
    def set_primary_phone_from_secondary(profile):
        if len(profile['secondary_phones']['secondary']) > 0:
            lst_secondary = list(profile['secondary_phones']['secondary'])
            profile['primary_phone'] = lst_secondary.pop(0)
            profile['secondary_phones']['secondary_size'] = len(lst_secondary)
            profile['secondary_phones']['secondary'] = lst_secondary
        return profile

    @staticmethod
    def set_validate_email(profile, primary_email=None, check_valid_email=True):
        lst_udt_email = list()
        udt_primary_email = None
        lst_email = []
        for e in profile.get("email"):
            valid_email = dict()
            valid_email["email"] = e
            valid_email["status"] = EmailValidateStatus.Uncheck
            if (
                    profile.get("primary_email")
                    and profile.get("primary_email").get("email") == e
            ):
                valid_email["status"] = profile.get("primary_email").get("status")
                if valid_email.get("status") != EmailValidateStatus.Uncheck:
                    valid_email["last_check"] = (
                        profile.get("primary_email").get("last_check")
                        if profile.get("primary_email").get("last_check")
                        else datetime.utcnow()
                    )
                else:
                    lst_email.append(e)
            else:
                if (
                        profile.get("secondary_emails")
                        and len(profile.get("secondary_emails")) > 0
                ):
                    se = next(
                        (
                            x
                            for x in profile.get("secondary_emails").get("secondary")
                            if x.get("email") == e
                        ),
                        None,
                    )
                    if se:
                        valid_email["status"] = se.get("status")
                        if valid_email.get("status") != EmailValidateStatus.Uncheck:
                            valid_email["last_check"] = (
                                se.get("last_check")
                                if se.get("last_check")
                                else datetime.utcnow()
                            )
                        else:
                            lst_email.append(e)
            if primary_email and primary_email == e:
                udt_primary_email = valid_email
                continue
            lst_udt_email.append(valid_email)
        secondary_emails = dict()
        secondary_emails["secondary_size"] = len(lst_udt_email)
        secondary_emails["secondary"] = lst_udt_email
        profile["secondary_emails"] = secondary_emails
        profile["primary_email"] = None
        if udt_primary_email is None:
            profile = CommonHelper.set_primary_email_from_secondary(profile)
        else:
            profile["primary_email"] = udt_primary_email
        return profile

    @staticmethod
    def set_phone_profile(profile, primary_phone=None):
        lst_udt_phone = list()
        udt_primary_phone = None
        for p in profile.get("phone_number"):
            phone_profile = dict()
            phone_profile["phone_number"] = p
            phone_profile["status"] = PhoneVerifyStatus.UnVerify
            phone_profile["last_verify"] = datetime.utcnow()

            # primary_phone = profile.get('primary_phone')
            if (
                    profile.get("primary_phone")
                    and profile.get("primary_phone").get("phone_number") == p
            ):
                phone_profile["status"] = profile.get("primary_phone").get("status")
                if phone_profile.get("status") != PhoneVerifyStatus.UnVerify:
                    phone_profile["last_verify"] = (
                        profile.get("primary_phone").get("last_verify")
                        if profile.get("primary_phone").get("last_verify")
                        else datetime.utcnow()
                    )
            if profile.get("secondary_phones"):
                for se in profile.get("secondary_phones").get("secondary"):
                    if se.get("phone_number") == p:
                        phone_profile["status"] = se.get("status")
                        if phone_profile.get("status") != PhoneVerifyStatus.UnVerify:
                            phone_profile["last_verify"] = (
                                se.get("last_verify")
                                if se.get("last_verify")
                                else datetime.utcnow()
                            )
                        break
            if primary_phone and primary_phone == p:
                udt_primary_phone = phone_profile
                continue
            lst_udt_phone.append(phone_profile)
        secondary_phones = dict()
        secondary_phones["secondary_size"] = len(lst_udt_phone)
        secondary_phones["secondary"] = lst_udt_phone
        profile["secondary_phones"] = secondary_phones
        profile["primary_phone"] = None
        if udt_primary_phone is None:
            profile = CommonHelper.set_primary_phone_from_secondary(profile)
        else:
            profile["primary_phone"] = udt_primary_phone
        return profile



def df_get_question_data():
    question_data = [
        {
            "id": 1,
            "question": "Có hợp đồng bảo hiểm trên 1 năm không?",
            "type": 1,
            "answer_template": {
                "answers": [
                    {
                        "int_result": 1,
                        "string_result": "Có"
                    },
                    {
                        "int_result": 0,
                        "string_result": "Không"
                    }
                ]
            },
            "group_code": "VPBANK_PERSONAL_QUESTION_1"
        },
        {
            "id": 2,
            "question": "Có sở hữu ô tô không?",
            "type": 1,
            "answer_template": {
                "answers": [
                    {
                        "int_result": 1,
                        "string_result": "Có"
                    },
                    {
                        "int_result": 0,
                        "string_result": "Không"
                    }
                ]
            },
            "group_code": "VPBANK_PERSONAL_QUESTION_1"
        },
        {
            "id": 3,
            "question": "Có sở hữu Bất động sản không?",
            "type": 1,
            "answer_template": {
                "answers": [
                    {
                        "int_result": 1,
                        "string_result": "Có"
                    },
                    {
                        "int_result": 0,
                        "string_result": "Không"
                    }
                ]
            },
            "group_code": "VPBANK_PERSONAL_QUESTION_1"
        }
    ]
    return question_data


def df_get_question_data_by_group_code(group_code):
    return [x for x in df_get_question_data() if x.get('group_code') == group_code]



from mobio.libs.profiling_mf import ProfileStructure
from mobio.libs.profiling_mf.common_helper import CommonHelper
from mobio.libs.profiling_mf.merge_fields.base_merge import BaseMerge, MergeListGroup


class MergePrimaryEmail(BaseMerge):
    def __check_suggest_email__(
        self, email, set_suggest_fields, set_unique_suggest_values
    ):
        suggest_email_1 = False
        if (
            ProfileStructure.EMAIL_1 not in set_suggest_fields
            and email not in set_unique_suggest_values
        ):
            suggest_email_1 = True
            set_suggest_fields.add(ProfileStructure.EMAIL_1)
            set_unique_suggest_values.add(email)
        return suggest_email_1

    def serialize_data(
        self,
        suggest_data,
        profile_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
        predict=None,
    ):
        primary_email = None
        email_status = None
        suggest_email_1 = False
        if profile_data:
            primary_email = profile_data.get(ProfileStructure.EMAIL)
            email_status = profile_data.get("status")
            if primary_email:
                suggest_email_1 = self.__check_suggest_email__(
                    email=primary_email,
                    set_suggest_fields=set_suggest_fields,
                    set_unique_suggest_values=set_unique_suggest_values,
                )
        field_value = self.__build_value__(
            value=primary_email, suggest=suggest_email_1, changealbe=False
        )
        field_value["status"] = email_status
        suggest_data[ProfileStructure.PRIMARY_EMAIL] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=True,
            editable=False,
            mergeable=True,
            order=1,
            group=MergeListGroup.INFORMATION,
            value=field_value,
            tooltip_i18n="i18n_email_merge_tooltip",
        )

    def set_filter_value(self, suggest_filter_data, profile_data):
        if profile_data:
            if type(profile_data) == dict:
                suggest_filter_data.add(profile_data.get(ProfileStructure.EMAIL))
            elif type(profile_data) == str and CommonHelper.validate_email(
                profile_data
            ):
                suggest_filter_data.add(profile_data)

    def serialize_origin_data(
        self,
        suggest_data,
        origin_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
    ):
        suggest_email_1 = False
        email_valid = None
        if CommonHelper.validate_email(origin_data):
            email_valid = origin_data.lower().strip()
            suggest_email_1 = self.__check_suggest_email__(
                email=email_valid,
                set_suggest_fields=set_suggest_fields,
                set_unique_suggest_values=set_unique_suggest_values,
            )
        field_value = self.__build_value__(
            value=email_valid, suggest=suggest_email_1, changealbe=False
        )
        suggest_data[ProfileStructure.PRIMARY_EMAIL] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=True,
            editable=False,
            mergeable=True,
            order=1,
            group=MergeListGroup.INFORMATION,
            value=field_value,
            tooltip_i18n="i18n_email_merge_tooltip",
        )

    def merge_data(self, target_data, source_data, field_key, is_master_data=False):
        if source_data:
            suggest = source_data.get("field_value").get("suggest")
            email = source_data.get("field_value").get("value")
            primary_email = (
                target_data.get(ProfileStructure.PRIMARY_EMAIL).get(
                    ProfileStructure.EMAIL
                )
                if target_data.get(ProfileStructure.PRIMARY_EMAIL)
                else None
            )

            set_email = set(
                target_data.get(ProfileStructure.EMAIL)
                if target_data.get(ProfileStructure.EMAIL)
                else []
            )
            # Nếu primary_email này ko được chọn, nó sẽ là secondary_email
            if primary_email:
                set_email.add(primary_email)

            if email and CommonHelper.validate_email(email):
                email = str(email).lower().strip()
                set_email.add(email)
                if field_key == ProfileStructure.PRIMARY_EMAIL and suggest:
                    primary_email = email
            target_data[ProfileStructure.EMAIL] = list(set_email)
            CommonHelper.set_validate_email(
                profile=target_data,
                primary_email=primary_email,
                check_valid_email=False,
            )


if __name__ == "__main__":
    old, new = MergePrimaryEmail().get_updated_data(
        data={

                "remove": [],
                "key": "primary_email",
                "display_type": "single_line",
                "add": [],
                "translate_key": "i18n_label_email",
                "format": None,
                "field_key": "primary_email",
                "field_property": 6,
                "field_name": "primary_email",
                "change": [{"from": "oumashu@gmail.com", "to": "oumashu@gmail1.com"}],

        }
    )
    print("old: {}, new: {}".format(old, new))



class CommonMerchant:
    MERGE_WHEN = 0.95
    PREFIX_DYNAMIC_FIELD = "_dyn"
    PREFIX_CRITERIA = "cri"


class ProfileStructure:
    THIRD_PARTY_INFO = "third_party_info"
    PROFILE_GROUP = "profile_group"
    BUSINESS_CASE_ID = "business_case_id"
    SOURCE_ID = "source_id"
    SOURCE_TYPE = "source_type"
    MERCHANT_ID = "merchant_id"
    PROFILE_ID = "profile_id"
    CREATED_TIME = "created_time"
    UPDATED_TIME = "updated_time"
    EMAIL = "email"
    EMAIL_1 = "email_1"
    EMAIL_2 = "email_2"
    PHONE_NUMBER = "phone_number"
    PHONE_NUMBER_1 = "phone_number_1"
    PHONE_NUMBER_2 = "phone_number_2"
    SOCIAL_USER = "social_user"
    SOCIAL_NAME = "social_name"
    SOCIAL_ID = "social_id"
    ID_SOCIAL = "id_social"
    SOCIAL_TYPE = "social_type"
    SOCIAL_ID_TYPE = "social_id_type"
    PEOPLE_ID = "people_id"
    IDENTIFY_CODE = "identify_code"
    DRIVER_LICENSE = "driver_license"
    PASSPORT = "passport"
    IS_COMPANY = "is_company"
    TAX_CODE = "tax_code"
    TAX_NAME = "tax_name"
    TAX_ADDRESS = "tax_address"
    LAT = "lat"
    LON = "lon"
    CREATED_ACCOUNT_TYPE = "created_account_type"
    BANK_ACC = "bank_acc"
    GENDER = "gender"
    BIRTHDAY = "birthday"
    BIRTH_YEAR = "birth_year"
    BIRTH_DATE = "birth_date"
    BIRTH_MONTH = "birth_month"
    BIRTH_MONTH_DATE = "birth_month_date"
    MARITAL_STATUS = "marital_status"
    NAME = "name"
    DISPLAY_NAME = "display_name"
    PASSWORD = "password"
    ADDRESS = "address"
    PROFILE_ADDRESS = "profile_address"
    PROVINCE_CODE = "province_code"
    DISTRICT_CODE = "district_code"
    WARD_CODE = "ward_code"
    FAX = "fax"
    OPERATION = "operation"
    JOB = "job"
    HOBBY = "hobby"
    TAGS = "tags"
    TAGS_SEARCH = "tags_search"
    PROFILE_TAGS = "profile_tags"
    RELIGIOUSNESS = "religiousness"
    NATION = "nation"
    AVATAR = "avatar"
    COMPANY = "company"
    FACE_ID = "face_id"
    INCOME_LOW_THRESHOLD = "income_low_threshold"
    INCOME_HIGH_THRESHOLD = "income_high_threshold"
    INCOME_TYPE = "income_type"
    BUDGET_LOW_THRESHOLD = "budget_low_threshold"
    BUDGET_HIGH_THRESHOLD = "budget_high_threshold"
    FREQUENTLY_DEMANDS = "frequently_demands"
    LST_PHONE_DELETED = "lst_phone_deleted"
    LST_EMAIL_DELETED = "lst_email_deleted"
    DEGREE = "DEGREE".lower()
    INCOME_FAMILY = "income_family"
    RELATIONSHIP_DATA = "relationship_data"
    RELATION_WITH_CHILDS = "relation_with_childs"
    CHILDS = "CHILDS".lower()
    NUMBER_CHILDS = "number_childs"
    CHILD_ID = "child_id"
    NTH = "nth"
    CUSTOMER_ID = "customer_id"
    CUSTOMER_CREATED_TIME = "customer_created_time"
    PARTNER_POINT = "partner_point"
    IS_STAFF_UPDATE = "is_staff_update"
    SOCIAL_TAGS = "social_tags"
    LENDING_LIMIT = "lending_limit"
    SOURCE = "source"
    PRIMARY_EMAIL = "primary_email"
    SECONDARY_EMAILS = "secondary_emails"
    PRIMARY_PHONE = "primary_phone"
    SECONDARY_PHONES = "secondary_phones"
    PREDICT = "predict"
    IS_NON_PROFILE = "is_non_profile"
    CARD = "card"
    CARDS = "cards"
    PUSH_ID = "push_id"
    POINT = "point"
    RANK_POINT = "rank_point"
    AGE = "age"
    DEVICES = "devices"
    TAGS_INTERACTIVE = "tags_interactive"
    SALARY = "salary"
    CIF_CODE = "cif"
    CLV = "clv"
    NUMBER_TRANSACTION = "number_transactions"
    PROFILE_IDENTIFY = "profile_identify"
    LAST_PAYMENT = "last_payment"
    MERGEABLE = "mergeable"
    DEVICE_TYPES = "device_types"
    ISP = 'isp'
    HIDDEN_AUDIENCE = "hidden_audience"


class ProfileByIdentifyStructure:
    MERCHANT_ID = "merchant_id"
    PROFILE_ID = "profile_id"
    IDENTIFY_VALUE = "identify_value"
    IDENTIFY_TYPE = "identify_type"
    IS_VERIFY = "is_verify"
    DATE_VERIFY = "date_verify"


class DeviceTypeStructure:
    DEVICE_TYPE = "device_type"
    DEVICE_NAME = "device_name"


class ProfileTagsStructure:
    ID = "id"
    TAG = "tag"
    TAG_TYPE = "tag_type"
    MERCHANT_ID = "merchant_id"


class TagInteractiveStructure:
    TAG_ID = "tag_id"
    INTERACTIVE_TOTAL = "interactive_total"
    INTERACTIVE_3_DAYS = "interactive_3_day"
    INTERACTIVE_7_DAYS = "interactive_7_day"
    INTERACTIVE_30_DAYS = "interactive_30_day"
    LAST_ACTION_TIME = "last_action_time"



from mobio.libs.profiling_mf.merge_fields.base_merge import BaseMerge, MergeListGroup


class MergeIdentifyCode(BaseMerge):
    def serialize_data(
        self,
        suggest_data,
        profile_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
        predict=None
    ):
        set_data = set()
        lst_suggest_data = []
        for a in profile_data:
            if a not in set_data:
                lst_suggest_data.append(self.__build_value__(value=a, suggest=True, predict=predict))
                set_data.add(a)
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=True,
            editable=True,
            mergeable=True,
            order=1,
            group=MergeListGroup.INFORMATION,
            value=lst_suggest_data,
        )

    def set_filter_value(self, suggest_filter_data, profile_data):
        pass

    def serialize_origin_data(
        self,
        suggest_data,
        origin_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
    ):
        set_data = set()
        lst_suggest_data = []
        origin_data = origin_data if origin_data is not None else []
        for a in origin_data:
            if a not in set_data:
                lst_suggest_data.append(self.__build_value__(value=a, suggest=True))
                set_data.add(a)
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=True,
            editable=True,
            mergeable=True,
            order=1,
            group=MergeListGroup.INFORMATION,
            value=lst_suggest_data,
        )

    def merge_data(self, target_data, source_data, field_key, is_master_data=False):
        target_data[field_key] = target_data.get(field_key) if target_data.get(field_key) else []
        set_data = set(target_data.get(field_key)) if not is_master_data else set()
        if source_data:
            for data in source_data.get('field_value'):
                suggest = data.get("suggest")
                value = str(data.get("value")).strip() if data.get("value") is not None else ""
                if suggest and value:
                    set_data.add(value)
        target_data[field_key] = list(set_data)



from mobio.libs.profiling_mf import ProfileStructure
from mobio.libs.profiling_mf.merge_fields.base_merge import BaseMerge, MergeListGroup
from mobio.libs.profiling_mf.profiling_common import ProfileHistoryChangeType
from mobio.libs.profiling_mf.profiling_schema import SocialUserSchema


class MergeSocialUser(BaseMerge):
    def serialize_data(
        self,
        suggest_data,
        profile_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
        predict=None,
    ):
        lst_social = []
        for social in profile_data:
            self.__add_social_user__(
                social_data=social,
                lst_social=lst_social,
                set_unique_suggest_values=set_unique_suggest_values,
            )
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=True,
            editable=False,
            mergeable=True,
            order=1,
            group=MergeListGroup.INFORMATION,
            value=lst_social,
        )

    def set_filter_value(self, suggest_filter_data, profile_data):
        pass

    def __add_social_user__(self, social_data, lst_social, set_unique_suggest_values):

        social_user = SocialUserSchema().load(social_data)
        unique_suggest_values = "{}:{}".format(
            social_user.get("social_id"), social_user.get("social_type")
        )
        suggest = (
            True if unique_suggest_values not in set_unique_suggest_values else False
        )
        lst_social.append(
            self.__build_value__(value=social_user, suggest=suggest, changealbe=False)
        )
        set_unique_suggest_values.add(unique_suggest_values)

    def serialize_origin_data(
        self,
        suggest_data,
        origin_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
    ):
        lst_social = []
        if origin_data:
            if type(origin_data) == list:
                for social in origin_data:
                    self.__add_social_user__(
                        social_data=social,
                        lst_social=lst_social,
                        set_unique_suggest_values=set_unique_suggest_values,
                    )
            elif type(origin_data) == dict:
                self.__add_social_user__(
                    social_data=origin_data,
                    lst_social=lst_social,
                    set_unique_suggest_values=set_unique_suggest_values,
                )
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=True,
            editable=False,
            mergeable=True,
            order=1,
            group=MergeListGroup.INFORMATION,
            value=lst_social,
        )

    def merge_data(self, target_data, source_data, field_key, is_master_data=False):
        lst_social_user = (
            target_data.get(ProfileStructure.SOCIAL_USER)
            if target_data.get(ProfileStructure.SOCIAL_USER)
            else []
        )
        if source_data:
            for source_social in source_data.get("field_value"):
                social_user = SocialUserSchema().load(source_social.get("value"))
                # lst_social_user = [
                #     x
                #     for x in target_data.get(ProfileStructure.SOCIAL_USER)
                #     if x.get("social_id") != social_user.get("social_id")
                #     and x.get("social_type") != social_user.get("social_type")
                # ]
                exists_social = next(
                    (
                        x
                        for x in lst_social_user
                        if x.get("social_id") == social_user.get("social_id")
                        and x.get("social_type") == social_user.get("social_type")
                    ),
                    None,
                )
                if not exists_social:
                    lst_social_user.append(social_user)
        target_data[ProfileStructure.SOCIAL_USER] = lst_social_user

    def get_add_data(self, data):
        if not data:
            data = dict()
        if ProfileHistoryChangeType.ADD not in data:
            data[ProfileHistoryChangeType.ADD] = []
        return [x.get("social_id") for x in data.get(ProfileHistoryChangeType.ADD)]

    def get_remove_data(self, data):
        if not data:
            data = dict()
        if ProfileHistoryChangeType.REMOVE not in data:
            data[ProfileHistoryChangeType.REMOVE] = []
        return [x.get("social_id") for x in data.get(ProfileHistoryChangeType.REMOVE)]



from mobio.libs.profiling_mf.merge_fields.base_merge import BaseMerge, MergeListGroup


class MergeProfileGroup(BaseMerge):
    def serialize_data(
        self,
        suggest_data,
        profile_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
        predict=None
    ):
        lst_suggest_profile_group = []
        for profile_group in profile_data:
            if profile_group:
                lst_suggest_profile_group.append(
                    self.__build_value__(value=str(profile_group), suggest=True)
                )
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=False,
            editable=False,
            mergeable=True,
            order=1,
            group=MergeListGroup.INFORMATION,
            value=lst_suggest_profile_group,
        )

    def set_filter_value(self, suggest_filter_data, profile_data):
        pass

    def serialize_origin_data(
        self,
        suggest_data,
        origin_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
    ):
        lst_suggest_profile_group = []
        origin_data = origin_data if origin_data else []
        for profile_group in origin_data:
            if profile_group:
                lst_suggest_profile_group.append(
                    self.__build_value__(value=str(profile_group), suggest=True)
                )
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=False,
            editable=False,
            mergeable=True,
            order=1,
            group=MergeListGroup.INFORMATION,
            value=lst_suggest_profile_group,
        )

    def merge_data(self, target_data, source_data, field_key, is_master_data=False):
        target_data[field_key] = target_data.get(field_key) if target_data.get(field_key) else []
        if source_data:
            for row in source_data.get("field_value"):
                suggest = row.get("suggest")
                value = str(row.get("value"))
                if (
                    suggest
                    and value is not None
                    and value not in target_data.get(field_key)
                ):
                    target_data[field_key].append(value)



from importlib import import_module
from mobio.libs.profiling_mf import ProfileStructure, CommonMerchant
from mobio.libs.profiling_mf.merge_fields.base_merge import BaseMerge


def dynamic_import_merge_v2(field_name: str):
    field_import = str(field_name)
    if field_import in [
        ProfileStructure.PHONE_NUMBER_1,
        ProfileStructure.PRIMARY_PHONE,
    ]:
        field_import = ProfileStructure.PRIMARY_PHONE
    elif field_import in [
        ProfileStructure.SECONDARY_PHONES,
        ProfileStructure.PHONE_NUMBER,
        ProfileStructure.PHONE_NUMBER_2,
    ]:
        field_import = ProfileStructure.SECONDARY_PHONES
    elif field_import in [ProfileStructure.EMAIL_1, ProfileStructure.PRIMARY_EMAIL]:
        field_import = ProfileStructure.PRIMARY_EMAIL
    elif field_import in [
        ProfileStructure.SECONDARY_EMAILS,
        ProfileStructure.EMAIL,
        ProfileStructure.EMAIL_2,
    ]:
        field_import = ProfileStructure.SECONDARY_EMAILS
    elif field_import.startswith(CommonMerchant.PREFIX_DYNAMIC_FIELD):
        field_import = "dynamic"
    elif field_import in [ProfileStructure.ADDRESS, ProfileStructure.PROFILE_ADDRESS]:
        field_import = ProfileStructure.ADDRESS
    elif field_import == "_id":
        field_import = "id"
    elif field_import in [ProfileStructure.TAGS, ProfileStructure.PROFILE_TAGS, ProfileStructure.TAGS_SEARCH]:
        field_import = "profile_tags"
    try:
        class_name = "Merge" + field_import.title().replace("_", "")
        if not field_import.startswith("."):
            field_import = ".merge_{}".format(field_import)
        merge_module = import_module(field_import, package="mobio.libs.profiling_mf.merge_fields")

        merge_class = getattr(merge_module, class_name)
        instance = merge_class()
    except (AttributeError, AssertionError, ModuleNotFoundError):
        raise ImportError(
            "{} is not part of our export collection!".format(field_import)
        )
    else:
        if not issubclass(merge_class, BaseMerge):
            raise ImportError(
                "We currently don't have {}, but you are welcome to send in the request for it!".format(
                    merge_class
                )
            )
    return instance



from mobio.libs.profiling_mf.merge_fields.base_merge import BaseMerge, MergeListGroup


class MergePushId(BaseMerge):
    def serialize_data(
        self,
        suggest_data,
        profile_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
        predict=None
    ):
        lst_suggest = []
        set_push_id = set()
        profile_data = profile_data if profile_data is not None else []
        for push_id in profile_data:
            if push_id.get("push_id") not in set_push_id:
                set_push_id.add(push_id.get("push_id"))
                lst_suggest.append(self.__build_value__(value=push_id, suggest=True, predict=predict))
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=False,
            editable=False,
            mergeable=True,
            order=1,
            group=MergeListGroup.OTHER,
            value=lst_suggest,
        )

    def set_filter_value(self, suggest_filter_data, profile_data):
        pass

    def serialize_origin_data(
        self,
        suggest_data,
        origin_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
    ):
        lst_suggest = []
        if type(origin_data) == list:
            set_push_id = set()
            for push_id in origin_data:
                if push_id.get("push_id") not in set_push_id:
                    set_push_id.add(push_id.get("push_id"))
                    lst_suggest.append(
                        self.__build_value__(value=push_id, suggest=True)
                    )
        elif type(origin_data) == dict:
            lst_suggest.append(self.__build_value__(value=origin_data, suggest=True))
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=False,
            editable=False,
            mergeable=True,
            order=1,
            group=MergeListGroup.OTHER,
            value=lst_suggest,
        )

    def merge_data(self, target_data, source_data, field_key, is_master_data=False):
        target_data[field_key] = target_data.get(field_key) if target_data.get(field_key) else []
        set_push_id = set([x.get("push_id") for x in target_data.get(field_key)])
        if source_data:
            for push_id in source_data.get("field_value"):
                suggest = push_id.get("suggest")
                value = push_id.get("value")
                if suggest and value.get("push_id") not in set_push_id:
                    target_data[field_key].append(value)
                    set_push_id.add(value.get("push_id"))



from mobio.libs.profiling_mf import ProfileStructure
from mobio.libs.profiling_mf.common_helper import CommonHelper
from mobio.libs.profiling_mf.merge_fields.base_merge import BaseMerge, MergeListGroup


class MergePrimaryPhone(BaseMerge):
    def __check_suggest_phone__(
        self, phone, set_suggest_fields, set_unique_suggest_values
    ):
        suggest_phone_1 = False
        if (
            ProfileStructure.PHONE_NUMBER_1 not in set_suggest_fields
            and phone not in set_unique_suggest_values
        ):
            suggest_phone_1 = True
            set_suggest_fields.add(ProfileStructure.PHONE_NUMBER_1)
            set_unique_suggest_values.add(phone)
        return suggest_phone_1

    def serialize_data(
        self,
        suggest_data,
        profile_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
        predict=None
    ):
        primary_phone = None
        phone_status = None
        suggest_phone_1 = False
        if profile_data:
            primary_phone = profile_data.get(ProfileStructure.PHONE_NUMBER)
            phone_status = profile_data.get("status")
            if primary_phone:
                suggest_phone_1 = self.__check_suggest_phone__(
                    phone=primary_phone,
                    set_suggest_fields=set_suggest_fields,
                    set_unique_suggest_values=set_unique_suggest_values,
                )
        field_value = self.__build_value__(
            value=primary_phone, suggest=suggest_phone_1, changealbe=False
        )
        field_value["status"] = phone_status
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=True,
            editable=False,
            mergeable=True,
            order=1,
            group=MergeListGroup.INFORMATION,
            value=field_value,
            tooltip_i18n="i18n_phone_merge_tooltip",
        )

    def set_filter_value(self, suggest_filter_data, profile_data):
        if profile_data:
            if type(profile_data) == str and CommonHelper().chuan_hoa_so_dien_thoai_v2(profile_data):
                suggest_filter_data.add(profile_data)
            elif type(profile_data) == dict and profile_data.get(
                ProfileStructure.PHONE_NUMBER
            ):
                suggest_filter_data.add(profile_data.get(ProfileStructure.PHONE_NUMBER))

    def serialize_origin_data(
        self,
        suggest_data,
        origin_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
    ):
        valid_phone = CommonHelper().chuan_hoa_so_dien_thoai_v2(origin_data)
        suggest_phone_1 = False
        if valid_phone:
            suggest_phone_1 = self.__check_suggest_phone__(
                phone=valid_phone,
                set_suggest_fields=set_suggest_fields,
                set_unique_suggest_values=set_unique_suggest_values,
            )
        field_value = self.__build_value__(
            value=valid_phone, suggest=suggest_phone_1, changealbe=False
        )
        suggest_data[field_key] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=True,
            editable=False,
            mergeable=True,
            order=1,
            group=MergeListGroup.INFORMATION,
            value=field_value,
            tooltip_i18n="i18n_phone_merge_tooltip",
        )

    def merge_data(self, target_data, source_data, field_key, is_master_data=False):
        if source_data:
            suggest = source_data.get("field_value").get("suggest")
            phone = source_data.get("field_value").get("value")
            primary_phone = (
                target_data.get(ProfileStructure.PRIMARY_PHONE).get(
                    ProfileStructure.PHONE_NUMBER
                )
                if target_data.get(ProfileStructure.PRIMARY_PHONE)
                else None
            )

            set_phone = set(
                target_data.get(ProfileStructure.PHONE_NUMBER)
                if target_data.get(ProfileStructure.PHONE_NUMBER)
                else []
            )
            # Nếu primary_phone này ko được chọn, nó sẽ là secondary_phone
            if primary_phone:
                set_phone.add(primary_phone)
            valid_phone = CommonHelper().chuan_hoa_so_dien_thoai_v2(phone)
            if valid_phone:
                set_phone.add(valid_phone)
                if field_key == ProfileStructure.PRIMARY_PHONE and suggest:
                    primary_phone = valid_phone
            target_data[ProfileStructure.PHONE_NUMBER] = list(set_phone)
            CommonHelper.set_phone_profile(
                profile=target_data, primary_phone=primary_phone
            )



from mobio.libs.profiling_mf import ProfileStructure
from mobio.libs.profiling_mf.common_helper import CommonHelper
from mobio.libs.profiling_mf.merge_fields.base_merge import BaseMerge, MergeListGroup
from mobio.libs.profiling_mf.profiling_common import ProfileHistoryChangeType


class MergeSecondaryEmails(BaseMerge):
    def serialize_data(
        self,
        suggest_data,
        profile_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
        predict=None,
    ):
        lst_email_2 = []
        if (
            type(profile_data) == dict
            and field_key == ProfileStructure.SECONDARY_EMAILS
        ):
            for email in profile_data.get("secondary"):
                # suggest = (
                #     True
                #     if email.get(ProfileStructure.EMAIL)
                #     not in set_unique_suggest_values
                #     else False
                # )
                status = email.get("status")
                value = email.get(ProfileStructure.EMAIL)
                field_value = self.__build_value__(
                    value=value, suggest=True, changealbe=False
                )
                field_value["status"] = status
                lst_email_2.append(field_value)
                # set_unique_suggest_values.add(email.get(ProfileStructure.EMAIL))
            suggest_data[field_key] = self.build_merge_data(
                translate_key=translate_key,
                field_property=field_property,
                display_type=display_type,
                displayable=True,
                editable=False,
                mergeable=True,
                order=1,
                group=MergeListGroup.INFORMATION,
                value=lst_email_2,
            )

    def set_filter_value(self, suggest_filter_data: set, profile_data):
        if type(profile_data) == dict:
            for email in profile_data.get("secondary"):
                suggest_filter_data.add(email.get(ProfileStructure.EMAIL))

    def serialize_origin_data(
        self,
        suggest_data,
        origin_data,
        set_suggest_fields,
        set_unique_suggest_values,
        field_key,
        field_property,
        display_type,
        translate_key,
    ):
        lst_email_2 = []
        if origin_data:
            if type(origin_data) == list:
                for email in origin_data:
                    if CommonHelper.validate_email(email):
                        field_value = self.__build_value__(
                            value=email, suggest=True, changealbe=False
                        )
                        lst_email_2.append(field_value)
            elif type(origin_data) == str:
                if CommonHelper.validate_email(origin_data):
                    field_value = self.__build_value__(
                        value=origin_data, suggest=True, changealbe=False
                    )
                    lst_email_2.append(field_value)
            else:
                print(
                    "key: {}, origin_data: {} is not valid".format(
                        field_key, origin_data
                    )
                )
        suggest_data[ProfileStructure.SECONDARY_EMAILS] = self.build_merge_data(
            translate_key=translate_key,
            field_property=field_property,
            display_type=display_type,
            displayable=True,
            editable=False,
            mergeable=True,
            order=1,
            group=MergeListGroup.INFORMATION,
            value=lst_email_2,
        )

    def merge_data(self, target_data, source_data, field_key, is_master_data=False):
        if source_data:
            primary_email = (
                target_data.get(ProfileStructure.PRIMARY_EMAIL).get(
                    ProfileStructure.EMAIL
                )
                if target_data.get(ProfileStructure.PRIMARY_EMAIL)
                else None
            )
            # normalize lại data
            target_data[ProfileStructure.EMAIL] = (
                target_data.get(ProfileStructure.EMAIL)
                if target_data.get(ProfileStructure.EMAIL)
                else []
            )
            # set_email = [] khi đây là master_data, nếu không thì set_data sẽ là tất cả email
            set_email = set(
                target_data.get(ProfileStructure.EMAIL) if not is_master_data else []
            )
            # add primary_email vào set_email để tránh bị mất primary_email
            if primary_email:
                set_email.add(primary_email)

            for source_email in source_data.get("field_value"):
                suggest = source_email.get("suggest")
                email = source_email.get("value")
                if email and CommonHelper.validate_email(email):
                    email = str(email).lower().strip()
                    if suggest:
                        set_email.add(email)

            target_data[ProfileStructure.EMAIL] = list(set_email)
            CommonHelper.set_validate_email(
                profile=target_data,
                primary_email=primary_email,
                check_valid_email=False,
            )

    def get_add_data(self, data):
        if not data:
            data = dict()
        if ProfileHistoryChangeType.ADD not in data:
            data[ProfileHistoryChangeType.ADD] = []
        return [
            x.get(ProfileStructure.EMAIL)
            for x in data.get(ProfileHistoryChangeType.ADD)
        ]

    def get_remove_data(self, data):
        if not data:
            data = dict()
        if ProfileHistoryChangeType.REMOVE not in data:
            data[ProfileHistoryChangeType.REMOVE] = []
        return [
            x.get(ProfileStructure.EMAIL)
            for x in data.get(ProfileHistoryChangeType.REMOVE)
        ]


if __name__ == "__main__":
    old = MergeSecondaryEmails().get_remove_data(
        data={
            "remove": [],
            "key": "secondary_emails",
            "display_type": "multi_line",
            "add": [{"status": 0, "email": "oumashu2@gmail.com"}],
            "translate_key": "label_other_email",
            "format": None,
            "field_key": "secondary_emails",
            "field_property": 6,
            "field_name": "secondary_emails",
            "change": [],
        }
    )
    new = MergeSecondaryEmails().get_add_data(
        data={
            "remove": [],
            "key": "secondary_emails",
            "display_type": "multi_line",
            "add": [{"status": 0, "email": "oumashu2@gmail.com"}],
            "translate_key": "label_other_email",
            "format": None,
            "field_key": "secondary_emails",
            "field_property": 6,
            "field_name": "secondary_emails",
            "change": [],
        }
    )
    print("old: {}, new: {}".format(old, new))
